package com.tourism.webadmin.app.website.controller;

import cn.dev33.satoken.annotation.SaIgnore;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.page.PageMethod;
import com.tourism.common.additional.model.FileUrlObject;
import com.tourism.common.additional.utils.JsonUtils;
import com.tourism.common.additional.utils.MapConvertUtils;
import com.tourism.common.additional.utils.StringUtils;
import com.tourism.common.additional.utils.UrlConvertUtils;
import com.tourism.common.core.constant.ErrorCodeEnum;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.core.util.MyPageUtil;
import com.tourism.webadmin.app.website.dto.TourismProjectToWebDto;
import com.tourism.webadmin.back.model.DirectoryInfo;
import com.tourism.webadmin.back.model.TourismFile;
import com.tourism.webadmin.back.model.TourismProject;
import com.tourism.webadmin.back.service.TourismProjectService;
import com.tourism.webadmin.back.vo.TourismProjectVo;
import com.tourism.webadmin.config.ApplicationConfig;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.*;

import static java.util.stream.Collectors.toList;

/**
 * 旅游项目管理操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "门户旅游项目管理接口")
@Slf4j
@RestController
@SaIgnore
@RequestMapping("/website/tourism/project")
@Validated
public class TourismProjectToWebController {

    @Autowired
    private TourismProjectService tourismProjectService;
    @Autowired
    private ApplicationConfig applicationConfig;

    /**
     * 列出符合过滤条件的旅游项目管理列表。
     *
     * @param tourismProjectDtoFilter 过滤对象。
     * @return 应答结果对象，包含查询结果集。
     */
    @GetMapping("/list")
    public ResponseResult<MyPageData<TourismProjectVo>> list(
            @Valid @ModelAttribute TourismProjectToWebDto tourismProjectDtoFilter) {
//        if(tourismProjectDtoFilter.getBelongTab() == null){
//            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST,"所属分类(belongTab)不能为空!");
//        }
        //如果belongTab小于1000并且大于10.则表明查询的是一级菜单
        if(tourismProjectDtoFilter.getBelongTab() !=null && tourismProjectDtoFilter.getBelongTab()>=10 && tourismProjectDtoFilter.getBelongTab()<1000){
            DirectoryInfo filter = new DirectoryInfo();
            filter.setParentId(tourismProjectDtoFilter.getBelongTab());
            filter.setEnable(1);
            List<DirectoryInfo> directoryInfoList = tourismProjectService.getDirectoryInfoList(filter, null);
            tourismProjectDtoFilter.setBelongTab(null);
            tourismProjectDtoFilter.setDirectoryInfoIds(directoryInfoList.stream().map(DirectoryInfo::getId).collect(toList()));
        }
        TourismProject tourismProjectFilter = MyModelUtil.copyTo(tourismProjectDtoFilter, TourismProject.class);
        //首页展示的为启用的内容
        tourismProjectFilter.setEnable(1);
        String orderBy = MyOrderParam.buildOrderBy(tourismProjectDtoFilter.getOrderParamList(), TourismProject.class);
        if (tourismProjectDtoFilter.getPageNum() != null && tourismProjectDtoFilter.getPageSize() != null ) {
            PageMethod.startPage(tourismProjectDtoFilter.getPageNum(), tourismProjectDtoFilter.getPageSize(), true);
        }
        List<TourismProject> tourismProjectList =
                tourismProjectService.getTourismProjectListWithRelation(tourismProjectFilter, orderBy);
        MyPageData<TourismProjectVo> tourismProjectVoMyPageData = MyPageUtil.makeResponseData(tourismProjectList, TourismProjectVo.class);

        List<TourismProjectVo> dataList = tourismProjectVoMyPageData.getDataList();
        //先把imgUrl由jaon转换为List<FileUrlObject>
        dataList.stream().forEach(item ->
        {
            List<String> urlList = UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), item.getId(), item.getHomeHotPicture(), "homeHotPicture");
            item.setHomeHotPicturesAfterConvert(urlList);
            List<String> arrayList = UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), item.getId(), item.getTourismUrl(), "tourismUrl");
            item.setTourismUrlsAfterConvert(arrayList);
            List<String> arrayList1 = UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), item.getId(), item.getTravelNotesBanner(), "travelNotesBanner");
            item.setTravelNotesBannerAfterConvert(arrayList1);
        });
        return ResponseResult.success(tourismProjectVoMyPageData);
    }

    /**
     * 查看指定旅游项目管理对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
//    @SaCheckPermission("tourismProject.view")
    @GetMapping("/detail")
    public ResponseResult<TourismProjectVo> detail(@NotNull(message = "所属目录(id)不能为空") Long id) {
        TourismProject tourismProject = tourismProjectService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourismProject == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourismProjectVo tourismProjectVo = MyModelUtil.copyTo(tourismProject, TourismProjectVo.class);
        List<String> arrayList = UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), tourismProjectVo.getId(), tourismProjectVo.getTourismUrl(), "tourismUrl");
        tourismProjectVo.setTourismUrlsAfterConvert(arrayList);
        List<String> arrayList1 = UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), tourismProjectVo.getId(), tourismProjectVo.getHomeHotPicture(), "homeHotPicture");
        tourismProjectVo.setHomeHotPicturesAfterConvert(arrayList1);
        List<String> arrayList2 = UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), tourismProjectVo.getId(), tourismProjectVo.getTravelNotesBanner(), "travelNotesBanner");
        tourismProjectVo.setTravelNotesBannerAfterConvert(arrayList2);
        if(tourismProjectVo.getTourismFile() != null){
            // 使用 Jackson 的 ObjectMapper 进行转换
            ObjectMapper objectMapper = new ObjectMapper();
            TourismFile tourismFile = objectMapper.convertValue(tourismProjectVo.getTourismFile(), TourismFile.class);
            if(StringUtils.isNotEmpty(tourismFile.getFileUrl())){
                List<FileUrlObject> fileUrlObjectList = JsonUtils.parseFileUrlArray(tourismFile.getFileUrl(), FileUrlObject.class);
                if(!CollectionUtils.isEmpty(fileUrlObjectList)){
                    ArrayList<String> arrayList3 = new ArrayList<>();
                    //遍历每个元素的数组对象,对元素的url对象进行拼接
                    for (FileUrlObject y : fileUrlObjectList) {
                        String imgUrlAfterConvert =
                                applicationConfig.getHostIpPort() + y.getDownloadUri() + "?id=" + tourismFile.getId() + "&fieldName=fileUrl&asImage=true&filename=" + y.getFilename();
                        arrayList3.add(imgUrlAfterConvert);
                    }
                    tourismFile.setFileUrlsAfterConvert(arrayList3);
                }
                tourismProjectVo.setTourismFile(MapConvertUtils.convertObjectToMap(tourismFile));
            }
        }
        return ResponseResult.success(tourismProjectVo);
    }
}
