package com.tourism.webadmin.app.wechat.controller;

import cn.dev33.satoken.annotation.SaIgnore;
import com.github.pagehelper.page.PageMethod;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.tourism.common.core.annotation.MyRequestBody;
import com.tourism.common.core.constant.ErrorCodeEnum;
import com.tourism.common.core.object.MyPageData;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.ResponseResult;
import com.tourism.common.core.util.MyCommonUtil;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.core.util.MyPageUtil;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.tourism.webadmin.back.dto.DeliveryAddressDto;
import com.tourism.webadmin.back.dto.DeliveryAddressWebDto;
import com.tourism.webadmin.back.model.DeliveryAddress;
import com.tourism.webadmin.back.service.DeliveryAddressService;
import com.tourism.webadmin.back.vo.DeliveryAddressVo;
import com.tourism.webadmin.config.ApplicationConfig;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Tag(name = "外卖地址管理管理接口")
@Slf4j
@RestController
@RequestMapping("/wechat/delivery/address")
@SaIgnore
public class WechatDeliveryAddressController {
    @Autowired
    private ApplicationConfig appConfig;
    @Autowired
    private DeliveryAddressService deliveryAddressService;

    /**
     * 新增外卖地址管理数据。
     *
     * @param deliveryAddressDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @SaIgnore
    @ApiOperationSupport(ignoreParameters = {"deliveryAddressDto.id"})
//    @SaCheckPermission("deliveryAddress.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody DeliveryAddressDto deliveryAddressDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(deliveryAddressDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        DeliveryAddress deliveryAddress = MyModelUtil.copyTo(deliveryAddressDto, DeliveryAddress.class);
        deliveryAddress = deliveryAddressService.saveNew(deliveryAddress);
        return ResponseResult.success(deliveryAddress.getId());
    }

    /**
     * 更新外卖地址管理数据。
     *
     * @param deliveryAddressDto 更新对象。
     * @return 应答结果对象。
     */
//    @SaCheckPermission("deliveryAddress.update")
    @SaIgnore
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody DeliveryAddressDto deliveryAddressDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(deliveryAddressDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        DeliveryAddress deliveryAddress = MyModelUtil.copyTo(deliveryAddressDto, DeliveryAddress.class);
        DeliveryAddress originalDeliveryAddress = deliveryAddressService.getById(deliveryAddress.getId());
        if (originalDeliveryAddress == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!deliveryAddressService.update(deliveryAddress, originalDeliveryAddress)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除外卖地址管理数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
//    @SaCheckPermission("deliveryAddress.delete")
    @SaIgnore
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除外卖地址管理数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
//    @SaCheckPermission("deliveryAddress.delete")
    @SaIgnore
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的外卖地址管理列表。
     *
     * @param deliveryAddressDtoFilter 过滤对象。
     * @return 应答结果对象，包含查询结果集。
     */
//    @SaCheckPermission("deliveryAddress.view")
    @SaIgnore
    @GetMapping("/list")
    public ResponseResult<MyPageData<DeliveryAddressVo>> list(
            @Valid @ModelAttribute DeliveryAddressWebDto deliveryAddressDtoFilter) {
        if (deliveryAddressDtoFilter != null) {
            PageMethod.startPage(deliveryAddressDtoFilter.getPageNum(), deliveryAddressDtoFilter.getPageSize(), true);
        }
        DeliveryAddress deliveryAddressFilter = MyModelUtil.copyTo(deliveryAddressDtoFilter, DeliveryAddress.class);
//        String orderBy = MyOrderParam.buildOrderBy(orderParam, DeliveryAddress.class);
        List<DeliveryAddress> deliveryAddressList =
                deliveryAddressService.getDeliveryAddressListWithRelation(deliveryAddressFilter, "create_time");
        deliveryAddressService.maskFieldDataList(deliveryAddressList, null);
        return ResponseResult.success(MyPageUtil.makeResponseData(deliveryAddressList, DeliveryAddressVo.class));
    }

    /**
     * 查看指定外卖地址管理对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
//    @SaCheckPermission("deliveryAddress.view")
    @SaIgnore
    @GetMapping("/view")
    public ResponseResult<DeliveryAddressVo> view(@RequestParam Long id) {
        DeliveryAddress deliveryAddress = deliveryAddressService.getByIdWithRelation(id, MyRelationParam.full());
        if (deliveryAddress == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        deliveryAddressService.maskFieldData(deliveryAddress, null);
        DeliveryAddressVo deliveryAddressVo = MyModelUtil.copyTo(deliveryAddress, DeliveryAddressVo.class);
        return ResponseResult.success(deliveryAddressVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        DeliveryAddress originalDeliveryAddress = deliveryAddressService.getById(id);
        if (originalDeliveryAddress == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!deliveryAddressService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
