package com.tourism.webadmin.app.wechat.controller;

import cn.dev33.satoken.annotation.SaIgnore;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.page.PageMethod;
import com.tourism.common.core.annotation.MyRequestBody;
import com.tourism.common.core.constant.ErrorCodeEnum;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.MyCommonUtil;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.core.util.MyPageUtil;
import com.tourism.webadmin.back.dto.DeliveryOrderDto;
import com.tourism.webadmin.back.dto.DeliveryOrderItemsDto;
import com.tourism.webadmin.back.dto.DeliveryOrderWebDto;
import com.tourism.webadmin.back.model.DeliveryOrder;
import com.tourism.webadmin.back.model.DeliveryOrderItems;
import com.tourism.webadmin.back.service.DeliveryOrderItemsService;
import com.tourism.webadmin.back.service.DeliveryOrderService;
import com.tourism.webadmin.back.vo.DeliveryOrderItemsVo;
import com.tourism.webadmin.back.vo.DeliveryOrderVo;
import com.tourism.webadmin.config.ApplicationConfig;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

@Tag(name = "外卖订单接口")
@Slf4j
@RestController
@RequestMapping("/wechat/delivery/order")
@SaIgnore
public class WechatDeliveryOrderController {

    @Autowired
    private ApplicationConfig appConfig;
    @Autowired
    private DeliveryOrderService deliveryOrderService;
    @Autowired
    private DeliveryOrderItemsService deliveryOrderItemsService;
    /**
     * 创建订单
     */
    @SaIgnore
    @PostMapping("/createOrder")
    public ResponseResult<Long> createOrder(@MyRequestBody DeliveryOrderDto deliveryOrderDto,
                              @MyRequestBody List<DeliveryOrderItemsDto> deliveryOrderItemsDtoList) {
        ResponseResult<Tuple2<DeliveryOrder, JSONObject>> verifyResult =
                this.doBusinessDataVerifyAndConvert(deliveryOrderDto, false, deliveryOrderItemsDtoList);
        if (!verifyResult.isSuccess()) {
            return ResponseResult.errorFrom(verifyResult);
        }
        Tuple2<DeliveryOrder, JSONObject> bizData = verifyResult.getData();
        DeliveryOrder deliveryOrder = bizData.getFirst();
        deliveryOrder = deliveryOrderService.saveNewWithRelation(deliveryOrder, bizData.getSecond());
        return ResponseResult.success(deliveryOrder.getId());
    }

    /**
     * 订单列表
     */
    @SaIgnore
    @GetMapping("/orderList")
    public ResponseResult<MyPageData<DeliveryOrderVo>> orderList(DeliveryOrderWebDto deliveryOrderDtoFilter) {
        if (deliveryOrderDtoFilter != null) {
            PageMethod.startPage(deliveryOrderDtoFilter.getPageNum(), deliveryOrderDtoFilter.getPageSize(), true);
        }
        DeliveryOrder deliveryOrderFilter = MyModelUtil.copyTo(deliveryOrderDtoFilter, DeliveryOrder.class);
//        String orderBy = MyOrderParam.buildOrderBy(orderParam, DeliveryOrder.class);
        List<DeliveryOrder> deliveryOrderList =
                deliveryOrderService.getDeliveryOrderListWithRelation(deliveryOrderFilter, "create_time");
        deliveryOrderService.maskFieldDataList(deliveryOrderList, null);

        //查询订单详情
        Set<Long> orderIds = deliveryOrderList.stream().map(e -> e.getId()).collect(Collectors.toSet());
        List<DeliveryOrderItems> items = deliveryOrderItemsService.getDeliveryOrderItemsVoList(orderIds);
        List<DeliveryOrderItemsVo> deliveryOrderItemsVos = MyModelUtil.copyCollectionTo(items, DeliveryOrderItemsVo.class);
        //做成map，方便匹配
        Map<Long, List<DeliveryOrderItemsVo>> collect1 = deliveryOrderItemsVos.stream().collect(Collectors.groupingBy(DeliveryOrderItemsVo::getOrderId));
        MyPageData<DeliveryOrderVo> pageData = MyPageUtil.makeResponseData(deliveryOrderList, DeliveryOrderVo.class);
        pageData.getDataList().forEach(e->{
            e.setItemList(collect1.get(e.getId()));
        });
        return ResponseResult.success(pageData);
    }

    /**
     * 订单详情
     */
    @SaIgnore
    @RequestMapping("/orderDetail")
    public ResponseResult<DeliveryOrderVo> orderDetail(@RequestParam Long id) {
        DeliveryOrder deliveryOrder = deliveryOrderService.getByIdWithRelation(id, MyRelationParam.full());
        if (deliveryOrder == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        deliveryOrderService.maskFieldData(deliveryOrder, null);
        DeliveryOrderVo deliveryOrderVo = MyModelUtil.copyTo(deliveryOrder, DeliveryOrderVo.class);
        DeliveryOrderItems build = DeliveryOrderItems.builder().orderId(deliveryOrderVo.getId()).build();
        List<DeliveryOrderItems> items = deliveryOrderItemsService.getDeliveryOrderItemsList(build, "create_time");
        //items转为vo
        deliveryOrderVo.setItemList(MyModelUtil.copyCollectionTo(items, DeliveryOrderItemsVo.class));
        return ResponseResult.success(deliveryOrderVo);
    }


    private ResponseResult<Tuple2<DeliveryOrder, JSONObject>> doBusinessDataVerifyAndConvert(
            DeliveryOrderDto deliveryOrderDto,
            boolean forUpdate,
            List<DeliveryOrderItemsDto> deliveryOrderItemsDtoList) {
        ErrorCodeEnum errorCode = ErrorCodeEnum.DATA_VALIDATED_FAILED;
        String errorMessage = MyCommonUtil.getModelValidationError(deliveryOrderDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(errorCode, errorMessage);
        }
        errorMessage = MyCommonUtil.getModelValidationError(deliveryOrderItemsDtoList);
        if (errorMessage != null) {
            return ResponseResult.error(errorCode, "参数 [deliveryOrderItemsDtoList] " + errorMessage);
        }
        // 全部关联从表数据的验证和转换
        JSONObject relationData = new JSONObject();
        CallResult verifyResult;
        // 下面是输入参数中，主表关联数据的验证。
        DeliveryOrder deliveryOrder = MyModelUtil.copyTo(deliveryOrderDto, DeliveryOrder.class);
        DeliveryOrder originalData;
        if (forUpdate && deliveryOrder != null) {
            originalData = deliveryOrderService.getById(deliveryOrder.getId());
            if (originalData == null) {
                return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
            }
            relationData.put("originalData", originalData);
        }
        // 处理主表的一对多关联 [DeliveryOrderItems]
        List<DeliveryOrderItems> deliveryOrderItemsList =
                MyModelUtil.copyCollectionTo(deliveryOrderItemsDtoList, DeliveryOrderItems.class);
        relationData.put("deliveryOrderItemsList", deliveryOrderItemsList);
        return ResponseResult.success(new Tuple2<>(deliveryOrder, relationData));
    }
}
