package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 公司富文本操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "公司富文本管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/companyContent")
public class CompanyContentController {

    @Autowired
    private CompanyContentService companyContentService;

    /**
     * 新增公司富文本数据。
     *
     * @param companyContentDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"companyContentDto.id"})
    @SaCheckPermission("companyContent.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody CompanyContentDto companyContentDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(companyContentDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        CompanyContent companyContent = MyModelUtil.copyTo(companyContentDto, CompanyContent.class);
        companyContent = companyContentService.saveNew(companyContent);
        return ResponseResult.success(companyContent.getId());
    }

    /**
     * 更新公司富文本数据。
     *
     * @param companyContentDto 更新对象。
     * @return 应答结果对象。
     */
    @SaCheckPermission("companyContent.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody CompanyContentDto companyContentDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(companyContentDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        CompanyContent companyContent = MyModelUtil.copyTo(companyContentDto, CompanyContent.class);
        CompanyContent originalCompanyContent = companyContentService.getById(companyContent.getId());
        if (originalCompanyContent == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!companyContentService.update(companyContent, originalCompanyContent)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除公司富文本数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("companyContent.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除公司富文本数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("companyContent.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的公司富文本列表。
     *
     * @param companyContentDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("companyContent.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<CompanyContentVo>> list(
            @MyRequestBody CompanyContentDto companyContentDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        CompanyContent companyContentFilter = MyModelUtil.copyTo(companyContentDtoFilter, CompanyContent.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, CompanyContent.class);
        List<CompanyContent> companyContentList =
                companyContentService.getCompanyContentListWithRelation(companyContentFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(companyContentList, CompanyContentVo.class));
    }

    /**
     * 查看指定公司富文本对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("companyContent.view")
    @GetMapping("/view")
    public ResponseResult<CompanyContentVo> view(@RequestParam Long id) {
        CompanyContent companyContent = companyContentService.getByIdWithRelation(id, MyRelationParam.full());
        if (companyContent == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        CompanyContentVo companyContentVo = MyModelUtil.copyTo(companyContent, CompanyContentVo.class);
        return ResponseResult.success(companyContentVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        CompanyContent originalCompanyContent = companyContentService.getById(id);
        if (originalCompanyContent == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!companyContentService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
