package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 公司富文本管理操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "公司富文本管理管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/contentCompany")
public class ContentCompanyController {

    @Autowired
    private ContentCompanyService contentCompanyService;

    /**
     * 新增公司富文本管理数据。
     *
     * @param contentCompanyDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"contentCompanyDto.id"})
    @SaCheckPermission("contentCompany.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody ContentCompanyDto contentCompanyDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(contentCompanyDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        ContentCompany contentCompany = MyModelUtil.copyTo(contentCompanyDto, ContentCompany.class);
        contentCompany = contentCompanyService.saveNew(contentCompany);
        return ResponseResult.success(contentCompany.getId());
    }

    /**
     * 更新公司富文本管理数据。
     *
     * @param contentCompanyDto 更新对象。
     * @return 应答结果对象。
     */
    @SaCheckPermission("contentCompany.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody ContentCompanyDto contentCompanyDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(contentCompanyDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        ContentCompany contentCompany = MyModelUtil.copyTo(contentCompanyDto, ContentCompany.class);
        ContentCompany originalContentCompany = contentCompanyService.getById(contentCompany.getId());
        if (originalContentCompany == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!contentCompanyService.update(contentCompany, originalContentCompany)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除公司富文本管理数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("contentCompany.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除公司富文本管理数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("contentCompany.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的公司富文本管理列表。
     *
     * @param contentCompanyDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("contentCompany.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<ContentCompanyVo>> list(
            @MyRequestBody ContentCompanyDto contentCompanyDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        ContentCompany contentCompanyFilter = MyModelUtil.copyTo(contentCompanyDtoFilter, ContentCompany.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, ContentCompany.class);
        List<ContentCompany> contentCompanyList =
                contentCompanyService.getContentCompanyListWithRelation(contentCompanyFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(contentCompanyList, ContentCompanyVo.class));
    }

    /**
     * 查看指定公司富文本管理对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("contentCompany.view")
    @GetMapping("/view")
    public ResponseResult<ContentCompanyVo> view(@RequestParam Long id) {
        ContentCompany contentCompany = contentCompanyService.getByIdWithRelation(id, MyRelationParam.full());
        if (contentCompany == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        ContentCompanyVo contentCompanyVo = MyModelUtil.copyTo(contentCompany, ContentCompanyVo.class);
        return ResponseResult.success(contentCompanyVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        ContentCompany originalContentCompany = contentCompanyService.getById(id);
        if (originalContentCompany == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!contentCompanyService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
