package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.alibaba.fastjson.JSONObject;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.tourism.webadmin.back.dto.DeliveryOrderDto;
import com.tourism.webadmin.back.dto.DeliveryOrderItemsDto;
import com.tourism.webadmin.back.model.DeliveryOrder;
import com.tourism.webadmin.back.model.DeliveryOrderItems;
import com.tourism.webadmin.back.service.DeliveryOrderService;
import com.tourism.webadmin.back.vo.DeliveryOrderVo;
import com.tourism.webadmin.config.ApplicationConfig;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.*;

/**
 * 外卖订单管理操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "外卖订单管理管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/deliveryOrder")
public class DeliveryOrderController {

    @Autowired
    private ApplicationConfig appConfig;
    @Autowired
    private DeliveryOrderService deliveryOrderService;

    /**
     * 新增外卖订单管理数据，及其关联的从表数据。
     *
     * @param deliveryOrderDto 新增主表对象。
     * @param deliveryOrderItemsDtoList 一对多外卖订单明细管理从表列表。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"deliveryOrderDto.id"})
    @SaCheckPermission("deliveryOrder.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(
            @MyRequestBody DeliveryOrderDto deliveryOrderDto,
            @MyRequestBody List<DeliveryOrderItemsDto> deliveryOrderItemsDtoList) {
        ResponseResult<Tuple2<DeliveryOrder, JSONObject>> verifyResult =
                this.doBusinessDataVerifyAndConvert(deliveryOrderDto, false, deliveryOrderItemsDtoList);
        if (!verifyResult.isSuccess()) {
            return ResponseResult.errorFrom(verifyResult);
        }
        Tuple2<DeliveryOrder, JSONObject> bizData = verifyResult.getData();
        DeliveryOrder deliveryOrder = bizData.getFirst();
        deliveryOrder = deliveryOrderService.saveNewWithRelation(deliveryOrder, bizData.getSecond());
        return ResponseResult.success(deliveryOrder.getId());
    }

    /**
     * 修改外卖订单管理数据，及其关联的从表数据。
     *
     * @param deliveryOrderDto 修改后的对象。
     * @param deliveryOrderItemsDtoList 一对多外卖订单明细管理从表列表。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"deliveryOrderDto.id"})
    @SaCheckPermission("deliveryOrder.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Long> update(
            @MyRequestBody DeliveryOrderDto deliveryOrderDto,
            @MyRequestBody List<DeliveryOrderItemsDto> deliveryOrderItemsDtoList) {
        String errorMessage;
        ResponseResult<Tuple2<DeliveryOrder, JSONObject>> verifyResult =
                this.doBusinessDataVerifyAndConvert(deliveryOrderDto, true, deliveryOrderItemsDtoList);
        if (!verifyResult.isSuccess()) {
            return ResponseResult.errorFrom(verifyResult);
        }
        Tuple2<DeliveryOrder, JSONObject> bizData = verifyResult.getData();
        DeliveryOrder originalDeliveryOrder = bizData.getSecond().getObject("originalData", DeliveryOrder.class);
        DeliveryOrder deliveryOrder = bizData.getFirst();
        if (!deliveryOrderService.updateWithRelation(deliveryOrder, originalDeliveryOrder, bizData.getSecond())) {
            errorMessage = "数据验证失败，[DeliveryOrder] 数据不存在!";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success(deliveryOrder.getId());
    }

    /**
     * 删除外卖订单管理数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("deliveryOrder.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除外卖订单管理数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("deliveryOrder.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的外卖订单管理列表。
     *
     * @param deliveryOrderDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("deliveryOrder.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<DeliveryOrderVo>> list(
            @MyRequestBody DeliveryOrderDto deliveryOrderDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        DeliveryOrder deliveryOrderFilter = MyModelUtil.copyTo(deliveryOrderDtoFilter, DeliveryOrder.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, DeliveryOrder.class);
        List<DeliveryOrder> deliveryOrderList =
                deliveryOrderService.getDeliveryOrderListWithRelation(deliveryOrderFilter, orderBy);
        deliveryOrderService.maskFieldDataList(deliveryOrderList, null);
        return ResponseResult.success(MyPageUtil.makeResponseData(deliveryOrderList, DeliveryOrderVo.class));
    }

    /**
     * 导入主表数据列表。
     *
     * @param importFile 上传的文件，目前仅仅支持xlsx和xls两种格式。
     * @return 应答结果对象。
     */
    @SaCheckPermission("deliveryOrder.import")
    @OperationLog(type = SysOperationLogType.IMPORT)
    @PostMapping("/import")
    public ResponseResult<Void> importBatch(
            @RequestParam Boolean skipHeader,
            @RequestParam("importFile") MultipartFile importFile) throws IOException {
        String filename = ImportUtil.saveImportFile(appConfig.getUploadFileBaseDir(), null, importFile);
        // 这里可以指定需要忽略导入的字段集合。如创建时间、创建人、更新时间、更新人、主键Id和逻辑删除，
        // 以及一些存在缺省值且无需导入的字段。其中主键字段和逻辑删除字段不需要在这里设置，批量插入逻辑会自动处理的。
        Set<String> ignoreFieldSet = new HashSet<>();
        ignoreFieldSet.add("createUserId");
        ignoreFieldSet.add("createTime");
        ignoreFieldSet.add("updateUserId");
        ignoreFieldSet.add("updateTime");
        List<ImportUtil.ImportHeaderInfo> headerInfoList = ImportUtil.makeHeaderInfoList(DeliveryOrder.class, ignoreFieldSet);
        // 下面是导入时需要注意的地方，如果我们缺省生成的代码，与实际情况存在差异，请手动修改。
        // 1. 头信息数据字段，我们只是根据当前的主表实体对象生成了缺省数组，开发者可根据实际情况，对headerInfoList进行修改。
        ImportUtil.ImportHeaderInfo[] headerInfos = headerInfoList.toArray(new ImportUtil.ImportHeaderInfo[]{});
        // 2. 这里需要根据实际情况决定，导入文件中第一行是否为中文头信息，如果是可以跳过。这里我们默认为true。
        // 这里根据自己的实际需求，为doImport的最后一个参数，传递需要进行字典转换的字段集合。
        // 注意，集合中包含需要翻译的Java字段名，如: gradeId。
        Set<String> translatedDictFieldSet = new HashSet<>();
        List<DeliveryOrder> dataList =
                ImportUtil.doImport(headerInfos, skipHeader, filename, DeliveryOrder.class, translatedDictFieldSet);
        deliveryOrderService.saveNewBatch(dataList, -1);
        return ResponseResult.success();
    }

    /**
     * 导出符合过滤条件的外卖订单管理列表。
     *
     * @param deliveryOrderDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @throws IOException 文件读写失败。
     */
    @SaCheckPermission("deliveryOrder.export")
    @OperationLog(type = SysOperationLogType.EXPORT, saveResponse = false)
    @PostMapping("/export")
    public void export(
            @MyRequestBody DeliveryOrderDto deliveryOrderDtoFilter,
            @MyRequestBody MyOrderParam orderParam) throws IOException {
        DeliveryOrder deliveryOrderFilter = MyModelUtil.copyTo(deliveryOrderDtoFilter, DeliveryOrder.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, DeliveryOrder.class);
        List<DeliveryOrder> resultList =
                deliveryOrderService.getDeliveryOrderListWithRelation(deliveryOrderFilter, orderBy);
        // 导出文件的标题数组
        // NOTE: 下面的代码中仅仅导出了主表数据，主表聚合计算数据和主表关联字典的数据。
        // 一对一从表数据的导出，可根据需要自行添加。如：headerMap.put("slaveFieldName.xxxField", "标题名称")
        Map<String, String> headerMap = new LinkedHashMap<>(21);
        headerMap.put("id", "主键");
        headerMap.put("deliveryTime", "配送时间");
        headerMap.put("deliveryAddress", "配送地址");
        headerMap.put("deliveryPhone", "收货电话");
        headerMap.put("deliveryRemark", "备注");
        headerMap.put("tableware", "餐具数量");
        headerMap.put("orderNo", "订单号");
        headerMap.put("orderTime", "下单时间");
        headerMap.put("payType", "支付方式");
        headerMap.put("orderAmount", "订单金额");
        headerMap.put("payAmount", "实付金额");
        headerMap.put("orderStatus", "订单状态");
        headerMap.put("customerId", "用户id");
        headerMap.put("refundStatus", "退款状态");
        headerMap.put("packCharge", "打包费");
        headerMap.put("deliveryFee", "配送费");
        headerMap.put("createUserId", "创建用户");
        headerMap.put("createTime", "创建时间");
        headerMap.put("updateUserId", "更新用户");
        headerMap.put("updateTime", "更新时间");
        headerMap.put("dataState", "删除标记(1: 正常 -1: 已删除)");
        ExportUtil.doExport(resultList, headerMap, "deliveryOrder.xlsx");
    }

    /**
     * 查看指定外卖订单管理对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("deliveryOrder.view")
    @GetMapping("/view")
    public ResponseResult<DeliveryOrderVo> view(@RequestParam Long id) {
        DeliveryOrder deliveryOrder = deliveryOrderService.getByIdWithRelation(id, MyRelationParam.full());
        if (deliveryOrder == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        deliveryOrderService.maskFieldData(deliveryOrder, null);
        DeliveryOrderVo deliveryOrderVo = MyModelUtil.copyTo(deliveryOrder, DeliveryOrderVo.class);
        return ResponseResult.success(deliveryOrderVo);
    }

    private ResponseResult<Tuple2<DeliveryOrder, JSONObject>> doBusinessDataVerifyAndConvert(
            DeliveryOrderDto deliveryOrderDto,
            boolean forUpdate,
            List<DeliveryOrderItemsDto> deliveryOrderItemsDtoList) {
        ErrorCodeEnum errorCode = ErrorCodeEnum.DATA_VALIDATED_FAILED;
        String errorMessage = MyCommonUtil.getModelValidationError(deliveryOrderDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(errorCode, errorMessage);
        }
        errorMessage = MyCommonUtil.getModelValidationError(deliveryOrderItemsDtoList);
        if (errorMessage != null) {
            return ResponseResult.error(errorCode, "参数 [deliveryOrderItemsDtoList] " + errorMessage);
        }
        // 全部关联从表数据的验证和转换
        JSONObject relationData = new JSONObject();
        CallResult verifyResult;
        // 下面是输入参数中，主表关联数据的验证。
        DeliveryOrder deliveryOrder = MyModelUtil.copyTo(deliveryOrderDto, DeliveryOrder.class);
        DeliveryOrder originalData;
        if (forUpdate && deliveryOrder != null) {
            originalData = deliveryOrderService.getById(deliveryOrder.getId());
            if (originalData == null) {
                return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
            }
            relationData.put("originalData", originalData);
        }
        // 处理主表的一对多关联 [DeliveryOrderItems]
        List<DeliveryOrderItems> deliveryOrderItemsList =
                MyModelUtil.copyCollectionTo(deliveryOrderItemsDtoList, DeliveryOrderItems.class);
        relationData.put("deliveryOrderItemsList", deliveryOrderItemsList);
        return ResponseResult.success(new Tuple2<>(deliveryOrder, relationData));
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        DeliveryOrder originalDeliveryOrder = deliveryOrderService.getById(id);
        if (originalDeliveryOrder == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!deliveryOrderService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
