package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 劳务富文本操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "劳务富文本管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/jobContent")
public class JobContentController {

    @Autowired
    private JobContentService jobContentService;

    /**
     * 新增劳务富文本数据。
     *
     * @param jobContentDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"jobContentDto.id"})
    @SaCheckPermission("jobContent.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody JobContentDto jobContentDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(jobContentDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        JobContent jobContent = MyModelUtil.copyTo(jobContentDto, JobContent.class);
        jobContent = jobContentService.saveNew(jobContent);
        return ResponseResult.success(jobContent.getId());
    }

    /**
     * 更新劳务富文本数据。
     *
     * @param jobContentDto 更新对象。
     * @return 应答结果对象。
     */
    @SaCheckPermission("jobContent.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody JobContentDto jobContentDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(jobContentDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        JobContent jobContent = MyModelUtil.copyTo(jobContentDto, JobContent.class);
        JobContent originalJobContent = jobContentService.getById(jobContent.getId());
        if (originalJobContent == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!jobContentService.update(jobContent, originalJobContent)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除劳务富文本数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("jobContent.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除劳务富文本数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("jobContent.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的劳务富文本列表。
     *
     * @param jobContentDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("jobContent.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<JobContentVo>> list(
            @MyRequestBody JobContentDto jobContentDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        JobContent jobContentFilter = MyModelUtil.copyTo(jobContentDtoFilter, JobContent.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, JobContent.class);
        List<JobContent> jobContentList = jobContentService.getJobContentListWithRelation(jobContentFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(jobContentList, JobContentVo.class));
    }

    /**
     * 查看指定劳务富文本对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("jobContent.view")
    @GetMapping("/view")
    public ResponseResult<JobContentVo> view(@RequestParam Long id) {
        JobContent jobContent = jobContentService.getByIdWithRelation(id, MyRelationParam.full());
        if (jobContent == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        JobContentVo jobContentVo = MyModelUtil.copyTo(jobContent, JobContentVo.class);
        return ResponseResult.success(jobContentVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        JobContent originalJobContent = jobContentService.getById(id);
        if (originalJobContent == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!jobContentService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
