package com.tourism.webadmin.app.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.tourism.webadmin.config.ApplicationConfig;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.*;

/**
 * 外卖购物车操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "外卖购物车管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/restaurantCart")
public class RestaurantCartController {

    @Autowired
    private ApplicationConfig appConfig;
    @Autowired
    private RestaurantCartService restaurantCartService;

    /**
     * 新增外卖购物车数据。
     *
     * @param restaurantCartDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"restaurantCartDto.id"})
    @SaCheckPermission("restaurantCart.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody RestaurantCartDto restaurantCartDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(restaurantCartDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        RestaurantCart restaurantCart = MyModelUtil.copyTo(restaurantCartDto, RestaurantCart.class);
        restaurantCart = restaurantCartService.saveNew(restaurantCart);
        return ResponseResult.success(restaurantCart.getId());
    }

    /**
     * 更新外卖购物车数据。
     *
     * @param restaurantCartDto 更新对象。
     * @return 应答结果对象。
     */
    @SaCheckPermission("restaurantCart.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody RestaurantCartDto restaurantCartDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(restaurantCartDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        RestaurantCart restaurantCart = MyModelUtil.copyTo(restaurantCartDto, RestaurantCart.class);
        RestaurantCart originalRestaurantCart = restaurantCartService.getById(restaurantCart.getId());
        if (originalRestaurantCart == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!restaurantCartService.update(restaurantCart, originalRestaurantCart)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除外卖购物车数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("restaurantCart.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除外卖购物车数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("restaurantCart.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的外卖购物车列表。
     *
     * @param restaurantCartDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("restaurantCart.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<RestaurantCartVo>> list(
            @MyRequestBody RestaurantCartDto restaurantCartDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        RestaurantCart restaurantCartFilter = MyModelUtil.copyTo(restaurantCartDtoFilter, RestaurantCart.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, RestaurantCart.class);
        List<RestaurantCart> restaurantCartList =
                restaurantCartService.getRestaurantCartListWithRelation(restaurantCartFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(restaurantCartList, RestaurantCartVo.class));
    }

    /**
     * 导入主表数据列表。
     *
     * @param importFile 上传的文件，目前仅仅支持xlsx和xls两种格式。
     * @return 应答结果对象。
     */
    @SaCheckPermission("restaurantCart.import")
    @OperationLog(type = SysOperationLogType.IMPORT)
    @PostMapping("/import")
    public ResponseResult<Void> importBatch(
            @RequestParam Boolean skipHeader,
            @RequestParam("importFile") MultipartFile importFile) throws IOException {
        String filename = ImportUtil.saveImportFile(appConfig.getUploadFileBaseDir(), null, importFile);
        // 这里可以指定需要忽略导入的字段集合。如创建时间、创建人、更新时间、更新人、主键Id和逻辑删除，
        // 以及一些存在缺省值且无需导入的字段。其中主键字段和逻辑删除字段不需要在这里设置，批量插入逻辑会自动处理的。
        Set<String> ignoreFieldSet = new HashSet<>();
        ignoreFieldSet.add("createUserId");
        ignoreFieldSet.add("createTime");
        ignoreFieldSet.add("updateUserId");
        ignoreFieldSet.add("updateTime");
        List<ImportUtil.ImportHeaderInfo> headerInfoList = ImportUtil.makeHeaderInfoList(RestaurantCart.class, ignoreFieldSet);
        // 下面是导入时需要注意的地方，如果我们缺省生成的代码，与实际情况存在差异，请手动修改。
        // 1. 头信息数据字段，我们只是根据当前的主表实体对象生成了缺省数组，开发者可根据实际情况，对headerInfoList进行修改。
        ImportUtil.ImportHeaderInfo[] headerInfos = headerInfoList.toArray(new ImportUtil.ImportHeaderInfo[]{});
        // 2. 这里需要根据实际情况决定，导入文件中第一行是否为中文头信息，如果是可以跳过。这里我们默认为true。
        // 这里根据自己的实际需求，为doImport的最后一个参数，传递需要进行字典转换的字段集合。
        // 注意，集合中包含需要翻译的Java字段名，如: gradeId。
        Set<String> translatedDictFieldSet = new HashSet<>();
        List<RestaurantCart> dataList =
                ImportUtil.doImport(headerInfos, skipHeader, filename, RestaurantCart.class, translatedDictFieldSet);
        restaurantCartService.saveNewBatch(dataList, -1);
        return ResponseResult.success();
    }

    /**
     * 导出符合过滤条件的外卖购物车列表。
     *
     * @param restaurantCartDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @throws IOException 文件读写失败。
     */
    @SaCheckPermission("restaurantCart.export")
    @OperationLog(type = SysOperationLogType.EXPORT, saveResponse = false)
    @PostMapping("/export")
    public void export(
            @MyRequestBody RestaurantCartDto restaurantCartDtoFilter,
            @MyRequestBody MyOrderParam orderParam) throws IOException {
        RestaurantCart restaurantCartFilter = MyModelUtil.copyTo(restaurantCartDtoFilter, RestaurantCart.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, RestaurantCart.class);
        List<RestaurantCart> resultList =
                restaurantCartService.getRestaurantCartListWithRelation(restaurantCartFilter, orderBy);
        // 导出文件的标题数组
        // NOTE: 下面的代码中仅仅导出了主表数据，主表聚合计算数据和主表关联字典的数据。
        // 一对一从表数据的导出，可根据需要自行添加。如：headerMap.put("slaveFieldName.xxxField", "标题名称")
        Map<String, String> headerMap = new LinkedHashMap<>(10);
        headerMap.put("id", "主键id");
        headerMap.put("userId", "用户id");
        headerMap.put("restaurantId", "餐馆id");
        headerMap.put("restaurantFoodId", "美食id");
        headerMap.put("count", "数量");
        headerMap.put("createUserId", "创建者Id");
        headerMap.put("createTime", "创建时间");
        headerMap.put("updateUserId", "更新者Id");
        headerMap.put("updateTime", "最后更新时间");
        headerMap.put("deletedFlag", "删除标记(1: 正常 -1: 已删除)");
        ExportUtil.doExport(resultList, headerMap, "restaurantCart.xlsx");
    }

    /**
     * 查看指定外卖购物车对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("restaurantCart.view")
    @GetMapping("/view")
    public ResponseResult<RestaurantCartVo> view(@RequestParam Long id) {
        RestaurantCart restaurantCart = restaurantCartService.getByIdWithRelation(id, MyRelationParam.full());
        if (restaurantCart == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        RestaurantCartVo restaurantCartVo = MyModelUtil.copyTo(restaurantCart, RestaurantCartVo.class);
        return ResponseResult.success(restaurantCartVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        RestaurantCart originalRestaurantCart = restaurantCartService.getById(id);
        if (originalRestaurantCart == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!restaurantCartService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
