package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.hutool.core.util.BooleanUtil;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.tourism.webadmin.config.ApplicationConfig;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.tourism.webadmin.upms.service.SysDeptService;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 餐馆美食类型操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "餐馆美食类型管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/restaurantFoodType")
public class RestaurantFoodTypeController {

    @Autowired
    private ApplicationConfig appConfig;
    @Autowired
    private RestaurantFoodTypeService restaurantFoodTypeService;
    @Autowired
    private SysDeptService sysDeptService;
    @Autowired
    private RestaurantInfoService restaurantService;


    /**
     * 新增餐馆美食类型数据。
     *
     * @param restaurantFoodTypeDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"restaurantFoodTypeDto.id", "restaurantFoodTypeDto.searchString"})
    @SaCheckPermission("restaurantFoodType.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody RestaurantFoodTypeDto restaurantFoodTypeDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(restaurantFoodTypeDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        RestaurantFoodType restaurantFoodType = MyModelUtil.copyTo(restaurantFoodTypeDto, RestaurantFoodType.class);
        // 验证关联Id的数据合法性
        CallResult callResult = restaurantFoodTypeService.verifyRelatedData(restaurantFoodType, null);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        restaurantFoodType = restaurantFoodTypeService.saveNew(restaurantFoodType);
        return ResponseResult.success(restaurantFoodType.getId());
    }

    /**
     * 更新餐馆美食类型数据。
     *
     * @param restaurantFoodTypeDto 更新对象。
     * @return 应答结果对象。
     */
    @ApiOperationSupport(ignoreParameters = {"restaurantFoodTypeDto.searchString"})
    @SaCheckPermission("restaurantFoodType.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody RestaurantFoodTypeDto restaurantFoodTypeDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(restaurantFoodTypeDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        RestaurantFoodType restaurantFoodType = MyModelUtil.copyTo(restaurantFoodTypeDto, RestaurantFoodType.class);
        RestaurantFoodType originalRestaurantFoodType = restaurantFoodTypeService.getById(restaurantFoodType.getId());
        if (originalRestaurantFoodType == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        // 验证关联Id的数据合法性
        CallResult callResult = restaurantFoodTypeService.verifyRelatedData(restaurantFoodType, originalRestaurantFoodType);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        if (!restaurantFoodTypeService.update(restaurantFoodType, originalRestaurantFoodType)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除餐馆美食类型数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("restaurantFoodType.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除餐馆美食类型数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("restaurantFoodType.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的餐馆美食类型列表。
     *
     * @param restaurantFoodTypeDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("restaurantFoodType.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<RestaurantFoodTypeVo>> list(
            @MyRequestBody RestaurantFoodTypeDto restaurantFoodTypeDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        TokenData tokenData = TokenData.takeFromRequest();
        //如果不是管理员角色的话，
        if(BooleanUtil.isFalse(tokenData.getIsAdmin())){
            List<Long> parentIds = new ArrayList<>();
            parentIds.add(tokenData.getDeptId());
            List<Long> allChildDeptIdByParentIds = sysDeptService.getAllChildDeptIdByParentIds(parentIds);
            // 使用HashSet构造函数将List转换为Set
            Set<Long> set = new HashSet<>(allChildDeptIdByParentIds);
            List<RestaurantInfo> restaurantInfoList = restaurantService.getInList("deptId", set);
            List<Long> restaurantIdList = restaurantInfoList.stream().map(RestaurantInfo::getId).collect(Collectors.toList());
            restaurantFoodTypeDtoFilter.setRestaurantIdList(restaurantIdList);
        }
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        RestaurantFoodType restaurantFoodTypeFilter = MyModelUtil.copyTo(restaurantFoodTypeDtoFilter, RestaurantFoodType.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, RestaurantFoodType.class);
        List<RestaurantFoodType> restaurantFoodTypeList =
                restaurantFoodTypeService.getRestaurantFoodTypeListWithRelation(restaurantFoodTypeFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(restaurantFoodTypeList, RestaurantFoodTypeVo.class));
    }

    /**
     * 导入主表数据列表。
     *
     * @param importFile 上传的文件，目前仅仅支持xlsx和xls两种格式。
     * @return 应答结果对象。
     */
    @SaCheckPermission("restaurantFoodType.import")
    @OperationLog(type = SysOperationLogType.IMPORT)
    @PostMapping("/import")
    public ResponseResult<Void> importBatch(
            @RequestParam Boolean skipHeader,
            @RequestParam("importFile") MultipartFile importFile) throws IOException {
        String filename = ImportUtil.saveImportFile(appConfig.getUploadFileBaseDir(), null, importFile);
        // 这里可以指定需要忽略导入的字段集合。如创建时间、创建人、更新时间、更新人、主键Id和逻辑删除，
        // 以及一些存在缺省值且无需导入的字段。其中主键字段和逻辑删除字段不需要在这里设置，批量插入逻辑会自动处理的。
        Set<String> ignoreFieldSet = new HashSet<>();
        ignoreFieldSet.add("createUserId");
        ignoreFieldSet.add("createTime");
        ignoreFieldSet.add("updateUserId");
        ignoreFieldSet.add("updateTime");
        List<ImportUtil.ImportHeaderInfo> headerInfoList = ImportUtil.makeHeaderInfoList(RestaurantFoodType.class, ignoreFieldSet);
        // 下面是导入时需要注意的地方，如果我们缺省生成的代码，与实际情况存在差异，请手动修改。
        // 1. 头信息数据字段，我们只是根据当前的主表实体对象生成了缺省数组，开发者可根据实际情况，对headerInfoList进行修改。
        ImportUtil.ImportHeaderInfo[] headerInfos = headerInfoList.toArray(new ImportUtil.ImportHeaderInfo[]{});
        // 2. 这里需要根据实际情况决定，导入文件中第一行是否为中文头信息，如果是可以跳过。这里我们默认为true。
        // 这里根据自己的实际需求，为doImport的最后一个参数，传递需要进行字典转换的字段集合。
        // 注意，集合中包含需要翻译的Java字段名，如: gradeId。
        Set<String> translatedDictFieldSet = new HashSet<>();
        List<RestaurantFoodType> dataList =
                ImportUtil.doImport(headerInfos, skipHeader, filename, RestaurantFoodType.class, translatedDictFieldSet);
        CallResult result = restaurantFoodTypeService.verifyImportList(dataList, translatedDictFieldSet);
        if (!result.isSuccess()) {
            // result中返回了具体的验证失败对象，如果需要返回更加详细的错误，可根据实际情况手动修改。
            return ResponseResult.errorFrom(result);
        }
        restaurantFoodTypeService.saveNewBatch(dataList, -1);
        return ResponseResult.success();
    }

    /**
     * 导出符合过滤条件的餐馆美食类型列表。
     *
     * @param restaurantFoodTypeDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @throws IOException 文件读写失败。
     */
    @SaCheckPermission("restaurantFoodType.export")
    @OperationLog(type = SysOperationLogType.EXPORT, saveResponse = false)
    @PostMapping("/export")
    public void export(
            @MyRequestBody RestaurantFoodTypeDto restaurantFoodTypeDtoFilter,
            @MyRequestBody MyOrderParam orderParam) throws IOException {
        RestaurantFoodType restaurantFoodTypeFilter = MyModelUtil.copyTo(restaurantFoodTypeDtoFilter, RestaurantFoodType.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, RestaurantFoodType.class);
        List<RestaurantFoodType> resultList =
                restaurantFoodTypeService.getRestaurantFoodTypeListWithRelation(restaurantFoodTypeFilter, orderBy);
        // 导出文件的标题数组
        // NOTE: 下面的代码中仅仅导出了主表数据，主表聚合计算数据和主表关联字典的数据。
        // 一对一从表数据的导出，可根据需要自行添加。如：headerMap.put("slaveFieldName.xxxField", "标题名称")
        Map<String, String> headerMap = new LinkedHashMap<>(11);
        headerMap.put("id", "主键id");
        headerMap.put("name", "种类名称");
        headerMap.put("restaurantIdDictMap.name", "餐馆id");
        headerMap.put("enable", "是否启用，0禁用，1启用");
        headerMap.put("showOrder", "显示顺序");
        headerMap.put("description", "描述");
        headerMap.put("createUserId", "创建用户");
        headerMap.put("createTime", "创建时间");
        headerMap.put("updateUserId", "更新用户");
        headerMap.put("updateTime", "更新时间");
        headerMap.put("dataState", "删除标记(1: 正常 -1: 已删除)");
        ExportUtil.doExport(resultList, headerMap, "restaurantFoodType.xlsx");
    }

    /**
     * 查看指定餐馆美食类型对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("restaurantFoodType.view")
    @GetMapping("/view")
    public ResponseResult<RestaurantFoodTypeVo> view(@RequestParam Long id) {
        RestaurantFoodType restaurantFoodType = restaurantFoodTypeService.getByIdWithRelation(id, MyRelationParam.full());
        if (restaurantFoodType == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        RestaurantFoodTypeVo restaurantFoodTypeVo = MyModelUtil.copyTo(restaurantFoodType, RestaurantFoodTypeVo.class);
        return ResponseResult.success(restaurantFoodTypeVo);
    }

    /**
     * 以字典形式返回全部餐馆美食类型数据集合。字典的键值为[id, name]。
     * 白名单接口，登录用户均可访问。
     *
     * @param filter 过滤对象。
     * @return 应答结果对象，包含的数据为 List<Map<String, String>>，map中包含两条记录，key的值分别是id和name，value对应具体数据。
     */
    @GetMapping("/listDict")
    public ResponseResult<List<Map<String, Object>>> listDict(@ParameterObject RestaurantFoodTypeDto filter) {

        TokenData tokenData = TokenData.takeFromRequest();
        //如果不是管理员角色的话，
        if(BooleanUtil.isFalse(tokenData.getIsAdmin())){
            List<Long> parentIds = new ArrayList<>();
            parentIds.add(tokenData.getDeptId());
            List<Long> allChildDeptIdByParentIds = sysDeptService.getAllChildDeptIdByParentIds(parentIds);
            // 使用HashSet构造函数将List转换为Set
            Set<Long> set = new HashSet<>(allChildDeptIdByParentIds);
            List<RestaurantInfo> restaurantInfoList = restaurantService.getInList("deptId", set);
            List<Long> restaurantIdList = restaurantInfoList.stream().map(RestaurantInfo::getId).collect(Collectors.toList());
            filter.setRestaurantIdList(restaurantIdList);
        }

        List<RestaurantFoodType> resultList =
                restaurantFoodTypeService.getRestaurantFoodTypeList(MyModelUtil.copyTo(filter, RestaurantFoodType.class),"");
        return ResponseResult.success(
                MyCommonUtil.toDictDataList(resultList, RestaurantFoodType::getId, RestaurantFoodType::getName));
    }

    /**
     * 根据字典Id集合，获取查询后的字典数据。
     *
     * @param dictIds 字典Id集合。
     * @return 应答结果对象，包含字典形式的数据集合。
     */
    @GetMapping("/listDictByIds")
    public ResponseResult<List<Map<String, Object>>> listDictByIds(@RequestParam List<Long> dictIds) {
        List<RestaurantFoodType> resultList = restaurantFoodTypeService.getInList(new HashSet<>(dictIds));
        return ResponseResult.success(
                MyCommonUtil.toDictDataList(resultList, RestaurantFoodType::getId, RestaurantFoodType::getName));
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        RestaurantFoodType originalRestaurantFoodType = restaurantFoodTypeService.getById(id);
        if (originalRestaurantFoodType == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!restaurantFoodTypeService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
