package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.dev33.satoken.annotation.SaIgnore;
import cn.hutool.core.util.BooleanUtil;
import cn.hutool.core.util.ReflectUtil;
import com.github.xiaoymin.knife4j.annotations.Ignore;
import com.tourism.common.additional.utils.StringUtils;
import com.tourism.common.core.upload.BaseUpDownloader;
import com.tourism.common.core.upload.UpDownloaderFactory;
import com.tourism.common.core.upload.UploadResponseInfo;
import com.tourism.common.core.upload.UploadStoreInfo;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.tourism.common.redis.cache.SessionCacheHelper;
import com.tourism.webadmin.config.ApplicationConfig;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.tourism.webadmin.upms.service.SysDeptService;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springdoc.core.annotations.ParameterObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import jakarta.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 餐馆信息操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "餐馆信息管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/restaurantInfo")
public class RestaurantInfoController {

    @Autowired
    private ApplicationConfig appConfig;
    @Autowired
    private SessionCacheHelper cacheHelper;
    @Autowired
    private UpDownloaderFactory upDownloaderFactory;
    @Autowired
    private RestaurantInfoService restaurantInfoService;
    @Autowired
    private RestaurantTypeService restaurantTypeService;
    @Autowired
    private SysDeptService sysDeptService;
    @Autowired
    private RestaurantInfoService restaurantService;

    /**
     * 新增餐馆信息数据。
     *
     * @param restaurantInfoDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"restaurantInfoDto.id", "restaurantInfoDto.searchString"})
    @SaCheckPermission("restaurantInfo.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody RestaurantInfoDto restaurantInfoDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(restaurantInfoDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        RestaurantInfo restaurantInfo = MyModelUtil.copyTo(restaurantInfoDto, RestaurantInfo.class);
        // 验证关联Id的数据合法性
        CallResult callResult = restaurantInfoService.verifyRelatedData(restaurantInfo, null);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        restaurantInfo = restaurantInfoService.saveNew(restaurantInfo);
        return ResponseResult.success(restaurantInfo.getId());
    }

    /**
     * 更新餐馆信息数据。
     *
     * @param restaurantInfoDto 更新对象。
     * @return 应答结果对象。
     */
    @ApiOperationSupport(ignoreParameters = {"restaurantInfoDto.searchString"})
    @SaCheckPermission("restaurantInfo.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody RestaurantInfoDto restaurantInfoDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(restaurantInfoDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        RestaurantInfo restaurantInfo = MyModelUtil.copyTo(restaurantInfoDto, RestaurantInfo.class);
        RestaurantInfo originalRestaurantInfo = restaurantInfoService.getById(restaurantInfo.getId());
        if (originalRestaurantInfo == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        // 验证关联Id的数据合法性
        CallResult callResult = restaurantInfoService.verifyRelatedData(restaurantInfo, originalRestaurantInfo);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        if (!restaurantInfoService.update(restaurantInfo, originalRestaurantInfo)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除餐馆信息数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("restaurantInfo.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除餐馆信息数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("restaurantInfo.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的餐馆信息列表。
     *
     * @param restaurantInfoDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("restaurantInfo.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<RestaurantInfoVo>> list(
            @MyRequestBody RestaurantInfoDto restaurantInfoDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        RestaurantInfo restaurantInfoFilter = MyModelUtil.copyTo(restaurantInfoDtoFilter, RestaurantInfo.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, RestaurantInfo.class);
        List<RestaurantInfo> restaurantInfoList =
                restaurantInfoService.getRestaurantInfoListWithRelation(restaurantInfoFilter, orderBy);
        MyPageData<RestaurantInfoVo> restaurantInfoVoMyPageData = MyPageUtil.makeResponseData(restaurantInfoList, RestaurantInfoVo.class);
        List<RestaurantInfoVo> dataList = restaurantInfoVoMyPageData.getDataList();
        Set<Long> restaurantTypeIdSet = new HashSet<>();
        for(RestaurantInfoVo i: dataList){
            if(StringUtils.isNotEmpty(i.getTypeId())){
                String[] split = i.getTypeId().split(",");
                for(String j: split){
                    restaurantTypeIdSet.add(Long.parseLong(j));
                }
            }
        }
        List<RestaurantType> inList = restaurantTypeService.getInList("id", restaurantTypeIdSet);
        if(!CollectionUtils.isEmpty(inList)) {
            Map<Long, String> collect = inList.stream().collect(Collectors.toMap(RestaurantType::getId, RestaurantType::getName));
            dataList.stream().forEach(i -> {
                if (StringUtils.isNotEmpty(i.getTypeId())) {
                    String[] split = i.getTypeId().split(",");
                    for (int j = 0; j < split.length; j++) {
                        if(j==0 && split.length == 1){
                            i.setTypeName(collect.get(Long.parseLong(split[j])));
                        } else if (j==0 && split.length>1) {
                            i.setTypeName(collect.get(Long.parseLong(split[j]))+",");
                        } else if(j==split.length-1){
                            i.setTypeName(i.getTypeName()+collect.get(Long.parseLong(split[j])));
                        } else {
                            i.setTypeName(i.getTypeName()+collect.get(Long.parseLong(split[j]))+",");
                        }
                    }

                }
            });
        }
        return ResponseResult.success(restaurantInfoVoMyPageData);
    }

    /**
     * 导入主表数据列表。
     *
     * @param importFile 上传的文件，目前仅仅支持xlsx和xls两种格式。
     * @return 应答结果对象。
     */
    @SaCheckPermission("restaurantInfo.import")
    @OperationLog(type = SysOperationLogType.IMPORT)
    @PostMapping("/import")
    public ResponseResult<Void> importBatch(
            @RequestParam Boolean skipHeader,
            @RequestParam("importFile") MultipartFile importFile) throws IOException {
        String filename = ImportUtil.saveImportFile(appConfig.getUploadFileBaseDir(), null, importFile);
        // 这里可以指定需要忽略导入的字段集合。如创建时间、创建人、更新时间、更新人、主键Id和逻辑删除，
        // 以及一些存在缺省值且无需导入的字段。其中主键字段和逻辑删除字段不需要在这里设置，批量插入逻辑会自动处理的。
        Set<String> ignoreFieldSet = new HashSet<>();
        ignoreFieldSet.add("createUserId");
        ignoreFieldSet.add("createTime");
        ignoreFieldSet.add("updateUserId");
        ignoreFieldSet.add("updateTime");
        List<ImportUtil.ImportHeaderInfo> headerInfoList = ImportUtil.makeHeaderInfoList(RestaurantInfo.class, ignoreFieldSet);
        // 下面是导入时需要注意的地方，如果我们缺省生成的代码，与实际情况存在差异，请手动修改。
        // 1. 头信息数据字段，我们只是根据当前的主表实体对象生成了缺省数组，开发者可根据实际情况，对headerInfoList进行修改。
        ImportUtil.ImportHeaderInfo[] headerInfos = headerInfoList.toArray(new ImportUtil.ImportHeaderInfo[]{});
        // 2. 这里需要根据实际情况决定，导入文件中第一行是否为中文头信息，如果是可以跳过。这里我们默认为true。
        // 这里根据自己的实际需求，为doImport的最后一个参数，传递需要进行字典转换的字段集合。
        // 注意，集合中包含需要翻译的Java字段名，如: gradeId。
        Set<String> translatedDictFieldSet = new HashSet<>();
        List<RestaurantInfo> dataList =
                ImportUtil.doImport(headerInfos, skipHeader, filename, RestaurantInfo.class, translatedDictFieldSet);
        CallResult result = restaurantInfoService.verifyImportList(dataList, translatedDictFieldSet);
        if (!result.isSuccess()) {
            // result中返回了具体的验证失败对象，如果需要返回更加详细的错误，可根据实际情况手动修改。
            return ResponseResult.errorFrom(result);
        }
        restaurantInfoService.saveNewBatch(dataList, -1);
        return ResponseResult.success();
    }

    /**
     * 导出符合过滤条件的餐馆信息列表。
     *
     * @param restaurantInfoDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @throws IOException 文件读写失败。
     */
    @SaCheckPermission("restaurantInfo.export")
    @OperationLog(type = SysOperationLogType.EXPORT, saveResponse = false)
    @PostMapping("/export")
    public void export(
            @MyRequestBody RestaurantInfoDto restaurantInfoDtoFilter,
            @MyRequestBody MyOrderParam orderParam) throws IOException {
        RestaurantInfo restaurantInfoFilter = MyModelUtil.copyTo(restaurantInfoDtoFilter, RestaurantInfo.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, RestaurantInfo.class);
        List<RestaurantInfo> resultList =
                restaurantInfoService.getRestaurantInfoListWithRelation(restaurantInfoFilter, orderBy);
        // 导出文件的标题数组
        // NOTE: 下面的代码中仅仅导出了主表数据，主表聚合计算数据和主表关联字典的数据。
        // 一对一从表数据的导出，可根据需要自行添加。如：headerMap.put("slaveFieldName.xxxField", "标题名称")
        Map<String, String> headerMap = new LinkedHashMap<>(22);
        headerMap.put("id", "主键id");
        headerMap.put("name", "店铺名称");
        headerMap.put("tag", "店铺标签");
        headerMap.put("typeIdDictMap.name", "店铺种类");
        headerMap.put("announcement", "店铺公告");
        headerMap.put("workTime", "工作时间段(例:8:00-19:00)");
        headerMap.put("adress", "店铺地址");
        headerMap.put("enableDictMap.name", "是否启用，0禁用，1启用");
        headerMap.put("url", "店铺图片");
        headerMap.put("environmentImage", "店铺环境图片");
        headerMap.put("contactInformation", "联系方式");
        headerMap.put("longitude", "经度");
        headerMap.put("latitude", "纬度");
        headerMap.put("totalSales", "销量");
        headerMap.put("contry", "所处国家");
        headerMap.put("createUserId", "创建用户");
        headerMap.put("city", "所处城市");
        headerMap.put("createTime", "创建时间");
        headerMap.put("updateUserId", "更新用户");
        headerMap.put("updateTime", "更新时间");
        headerMap.put("dataState", "删除标记(1: 正常 -1: 已删除)");
        headerMap.put("deptId", "用户所在部门Id");
        ExportUtil.doExport(resultList, headerMap, "restaurantInfo.xlsx");
    }

    /**
     * 查看指定餐馆信息对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("restaurantInfo.view")
    @GetMapping("/view")
    public ResponseResult<RestaurantInfoVo> view(@RequestParam Long id) {
        RestaurantInfo restaurantInfo = restaurantInfoService.getByIdWithRelation(id, MyRelationParam.full());
        if (restaurantInfo == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        RestaurantInfoVo restaurantInfoVo = MyModelUtil.copyTo(restaurantInfo, RestaurantInfoVo.class);
        if(StringUtils.isNotEmpty(restaurantInfoVo.getTypeId())) {
            restaurantInfoVo.setTypeIdList(Arrays.asList(restaurantInfoVo.getTypeId().split(",")));
        }
        return ResponseResult.success(restaurantInfoVo);
    }

    /**
     * 附件文件下载。
     * 这里将图片和其他类型的附件文件放到不同的父目录下，主要为了便于今后图片文件的迁移。
     *
     * @param id 附件所在记录的主键Id。
     * @param fieldName 附件所属的字段名。
     * @param filename  文件名。如果没有提供该参数，就从当前记录的指定字段中读取。
     * @param asImage   下载文件是否为图片。
     * @param response  Http 应答对象。
     */
//    @SaCheckPermission("restaurantInfo.view")
    @SaIgnore
    @OperationLog(type = SysOperationLogType.DOWNLOAD, saveResponse = false)
    @GetMapping("/download")
    public void download(
            @RequestParam(required = false) Long id,
            @RequestParam String fieldName,
            @RequestParam String filename,
            @RequestParam Boolean asImage,
            HttpServletResponse response) {
        if (MyCommonUtil.existBlankArgument(fieldName, filename, asImage)) {
            response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
            return;
        }
        // 使用try来捕获异常，是为了保证一旦出现异常可以返回500的错误状态，便于调试。
        // 否则有可能给前端返回的是200的错误码。
        try {
            // 如果请求参数中没有包含主键Id，就判断该文件是否为当前session上传的。
            if (id == null) {
                if (!cacheHelper.existSessionUploadFile(filename)) {
                    ResponseResult.output(HttpServletResponse.SC_FORBIDDEN);
                    return;
                }
            } else {
                RestaurantInfo restaurantInfo = restaurantInfoService.getById(id);
                if (restaurantInfo == null) {
                    ResponseResult.output(HttpServletResponse.SC_NOT_FOUND);
                    return;
                }
                String fieldJsonData = (String) ReflectUtil.getFieldValue(restaurantInfo, fieldName);
                if (fieldJsonData == null && !cacheHelper.existSessionUploadFile(filename)) {
                    ResponseResult.output(HttpServletResponse.SC_BAD_REQUEST);
                    return;
                }
                if (!BaseUpDownloader.containFile(fieldJsonData, filename)
                        && !cacheHelper.existSessionUploadFile(filename)) {
                    ResponseResult.output(HttpServletResponse.SC_FORBIDDEN);
                    return;
                }
            }
            UploadStoreInfo storeInfo = MyModelUtil.getUploadStoreInfo(RestaurantInfo.class, fieldName);
            if (!storeInfo.isSupportUpload()) {
                ResponseResult.output(HttpServletResponse.SC_NOT_IMPLEMENTED,
                        ResponseResult.error(ErrorCodeEnum.INVALID_UPLOAD_FIELD));
                return;
            }
            BaseUpDownloader upDownloader = upDownloaderFactory.get(storeInfo.getStoreType());
            upDownloader.doDownload(appConfig.getUploadFileBaseDir(),
                    RestaurantInfo.class.getSimpleName(), fieldName, filename, asImage, response);
        } catch (Exception e) {
            response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
            log.error(e.getMessage(), e);
        }
    }

    /**
     * 文件上传操作。
     *
     * @param fieldName  上传文件名。
     * @param asImage    是否作为图片上传。如果是图片，今后下载的时候无需权限验证。否则就是附件上传，下载时需要权限验证。
     * @param uploadFile 上传文件对象。
     */
    @SaCheckPermission("restaurantInfo.view")
    @OperationLog(type = SysOperationLogType.UPLOAD, saveResponse = false)
    @PostMapping("/upload")
    public void upload(
            @RequestParam String fieldName,
            @RequestParam Boolean asImage,
            @RequestParam("uploadFile") MultipartFile uploadFile) throws IOException {
        UploadStoreInfo storeInfo = MyModelUtil.getUploadStoreInfo(RestaurantInfo.class, fieldName);
        // 这里就会判断参数中指定的字段，是否支持上传操作。
        if (!storeInfo.isSupportUpload()) {
            ResponseResult.output(HttpServletResponse.SC_FORBIDDEN,
                    ResponseResult.error(ErrorCodeEnum.INVALID_UPLOAD_FIELD));
            return;
        }
        // 根据字段注解中的存储类型，通过工厂方法获取匹配的上传下载实现类，从而解耦。
        BaseUpDownloader upDownloader = upDownloaderFactory.get(storeInfo.getStoreType());
        UploadResponseInfo responseInfo = upDownloader.doUpload(null,
                appConfig.getUploadFileBaseDir(), RestaurantInfo.class.getSimpleName(), fieldName, asImage, uploadFile);
        if (Boolean.TRUE.equals(responseInfo.getUploadFailed())) {
            ResponseResult.output(HttpServletResponse.SC_FORBIDDEN,
                    ResponseResult.error(ErrorCodeEnum.UPLOAD_FAILED, responseInfo.getErrorMessage()));
            return;
        }
        cacheHelper.putSessionUploadFile(responseInfo.getFilename());
        ResponseResult.output(ResponseResult.success(responseInfo));
    }

    /**
     * 以字典形式返回全部餐馆信息数据集合。字典的键值为[id, name]。
     * 白名单接口，登录用户均可访问。
     *
     * @param filter 过滤对象。
     * @return 应答结果对象，包含的数据为 List<Map<String, String>>，map中包含两条记录，key的值分别是id和name，value对应具体数据。
     */
    @GetMapping("/listDict")
    public ResponseResult<List<Map<String, Object>>> listDict(@ParameterObject RestaurantInfoDto filter) {

        List<RestaurantInfo> resultList =
                restaurantInfoService.getRestaurantInfoList(MyModelUtil.copyTo(filter, RestaurantInfo.class),"");
        return ResponseResult.success(
                MyCommonUtil.toDictDataList(resultList, RestaurantInfo::getId, RestaurantInfo::getName));
    }

    /**
     * 根据字典Id集合，获取查询后的字典数据。
     *
     * @param dictIds 字典Id集合。
     * @return 应答结果对象，包含字典形式的数据集合。
     */
    @GetMapping("/listDictByIds")
    public ResponseResult<List<Map<String, Object>>> listDictByIds(@RequestParam List<Long> dictIds) {
        List<RestaurantInfo> resultList = restaurantInfoService.getInList(new HashSet<>(dictIds));
        return ResponseResult.success(
                MyCommonUtil.toDictDataList(resultList, RestaurantInfo::getId, RestaurantInfo::getName));
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        RestaurantInfo originalRestaurantInfo = restaurantInfoService.getById(id);
        if (originalRestaurantInfo == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!restaurantInfoService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
