package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.github.pagehelper.page.PageMethod;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.tourism.common.core.annotation.MyRequestBody;
import com.tourism.common.core.constant.ErrorCodeEnum;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.tourism.webadmin.back.dto.TourBookInfoDto;
import com.tourism.webadmin.back.model.TourBookInfo;
import com.tourism.webadmin.back.service.TourBookInfoService;
import com.tourism.webadmin.back.vo.TourBookInfoVo;
import com.tourism.webadmin.config.ApplicationConfig;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.*;

/**
 * 门户预定管理操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "门户预定管理管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourBookInfo")
public class TourBookInfoController {

    @Autowired
    private ApplicationConfig appConfig;
    @Autowired
    private TourBookInfoService tourBookInfoService;

    /**
     * 新增门户预定管理数据。
     *
     * @param tourBookInfoDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"tourBookInfoDto.id", "tourBookInfoDto.searchString"})
    @SaCheckPermission("tourBookInfo.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody TourBookInfoDto tourBookInfoDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourBookInfoDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourBookInfo tourBookInfo = MyModelUtil.copyTo(tourBookInfoDto, TourBookInfo.class);
        // 验证关联Id的数据合法性
        CallResult callResult = tourBookInfoService.verifyRelatedData(tourBookInfo, null);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        tourBookInfo = tourBookInfoService.saveNew(tourBookInfo);
        return ResponseResult.success(tourBookInfo.getId());
    }

    /**
     * 更新门户预定管理数据。
     *
     * @param tourBookInfoDto 更新对象。
     * @return 应答结果对象。
     */
    @ApiOperationSupport(ignoreParameters = {"tourBookInfoDto.searchString"})
    @SaCheckPermission("tourBookInfo.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody TourBookInfoDto tourBookInfoDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourBookInfoDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourBookInfo tourBookInfo = MyModelUtil.copyTo(tourBookInfoDto, TourBookInfo.class);
        TourBookInfo originalTourBookInfo = tourBookInfoService.getById(tourBookInfo.getId());
        if (originalTourBookInfo == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        // 验证关联Id的数据合法性
        CallResult callResult = tourBookInfoService.verifyRelatedData(tourBookInfo, originalTourBookInfo);
        if (!callResult.isSuccess()) {
            return ResponseResult.errorFrom(callResult);
        }
        if (!tourBookInfoService.update(tourBookInfo, originalTourBookInfo)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除门户预定管理数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourBookInfo.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除门户预定管理数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourBookInfo.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的门户预定管理列表。
     *
     * @param tourBookInfoDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourBookInfo.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourBookInfoVo>> list(
            @MyRequestBody TourBookInfoDto tourBookInfoDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourBookInfo tourBookInfoFilter = MyModelUtil.copyTo(tourBookInfoDtoFilter, TourBookInfo.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourBookInfo.class);
        List<TourBookInfo> tourBookInfoList =
                tourBookInfoService.getTourBookInfoListWithRelation(tourBookInfoFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourBookInfoList, TourBookInfoVo.class));
    }

    /**
     * 导入主表数据列表。
     *
     * @param importFile 上传的文件，目前仅仅支持xlsx和xls两种格式。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourBookInfo.import")
    @OperationLog(type = SysOperationLogType.IMPORT)
    @PostMapping("/import")
    public ResponseResult<Void> importBatch(
            @RequestParam Boolean skipHeader,
            @RequestParam("importFile") MultipartFile importFile) throws IOException {
        String filename = ImportUtil.saveImportFile(appConfig.getUploadFileBaseDir(), null, importFile);
        // 这里可以指定需要忽略导入的字段集合。如创建时间、创建人、更新时间、更新人、主键Id和逻辑删除，
        // 以及一些存在缺省值且无需导入的字段。其中主键字段和逻辑删除字段不需要在这里设置，批量插入逻辑会自动处理的。
        Set<String> ignoreFieldSet = new HashSet<>();
        ignoreFieldSet.add("createUserId");
        ignoreFieldSet.add("createTime");
        ignoreFieldSet.add("updateUserId");
        ignoreFieldSet.add("updateTime");
        List<ImportUtil.ImportHeaderInfo> headerInfoList = ImportUtil.makeHeaderInfoList(TourBookInfo.class, ignoreFieldSet);
        // 下面是导入时需要注意的地方，如果我们缺省生成的代码，与实际情况存在差异，请手动修改。
        // 1. 头信息数据字段，我们只是根据当前的主表实体对象生成了缺省数组，开发者可根据实际情况，对headerInfoList进行修改。
        ImportUtil.ImportHeaderInfo[] headerInfos = headerInfoList.toArray(new ImportUtil.ImportHeaderInfo[]{});
        // 2. 这里需要根据实际情况决定，导入文件中第一行是否为中文头信息，如果是可以跳过。这里我们默认为true。
        // 这里根据自己的实际需求，为doImport的最后一个参数，传递需要进行字典转换的字段集合。
        // 注意，集合中包含需要翻译的Java字段名，如: gradeId。
        Set<String> translatedDictFieldSet = new HashSet<>();
        List<TourBookInfo> dataList =
                ImportUtil.doImport(headerInfos, skipHeader, filename, TourBookInfo.class, translatedDictFieldSet);
        CallResult result = tourBookInfoService.verifyImportList(dataList, translatedDictFieldSet);
        if (!result.isSuccess()) {
            // result中返回了具体的验证失败对象，如果需要返回更加详细的错误，可根据实际情况手动修改。
            return ResponseResult.errorFrom(result);
        }
        tourBookInfoService.saveNewBatch(dataList, -1);
        return ResponseResult.success();
    }

    /**
     * 导出符合过滤条件的门户预定管理列表。
     *
     * @param tourBookInfoDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @throws IOException 文件读写失败。
     */
    @SaCheckPermission("tourBookInfo.export")
    @OperationLog(type = SysOperationLogType.EXPORT, saveResponse = false)
    @PostMapping("/export")
    public void export(
            @MyRequestBody TourBookInfoDto tourBookInfoDtoFilter,
            @MyRequestBody MyOrderParam orderParam) throws IOException {
        TourBookInfo tourBookInfoFilter = MyModelUtil.copyTo(tourBookInfoDtoFilter, TourBookInfo.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourBookInfo.class);
        List<TourBookInfo> resultList =
                tourBookInfoService.getTourBookInfoListWithRelation(tourBookInfoFilter, orderBy);
        // 导出文件的标题数组
        // NOTE: 下面的代码中仅仅导出了主表数据，主表聚合计算数据和主表关联字典的数据。
        // 一对一从表数据的导出，可根据需要自行添加。如：headerMap.put("slaveFieldName.xxxField", "标题名称")
        Map<String, String> headerMap = new LinkedHashMap<>(11);
        headerMap.put("id", "主键id");
        headerMap.put("bookMobile", "预定电话");
        headerMap.put("bookName", "预定姓名");
        headerMap.put("bookIp", "预定的ip");
        headerMap.put("bookTime", "预定时间");
        headerMap.put("isHandleDictMap.name", "是否处理(0:未处理 1:已处理)");
        headerMap.put("createUserId", "创建者Id");
        headerMap.put("createTime", "创建时间");
        headerMap.put("updateUserId", "更新者Id");
        headerMap.put("updateTime", "最后更新时间");
        headerMap.put("deletedFlag", "删除标记(1: 正常 -1: 已删除)");
        ExportUtil.doExport(resultList, headerMap, "tourBookInfo.xlsx");
    }

    /**
     * 查看指定门户预定管理对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourBookInfo.view")
    @GetMapping("/view")
    public ResponseResult<TourBookInfoVo> view(@RequestParam Long id) {
        TourBookInfo tourBookInfo = tourBookInfoService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourBookInfo == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourBookInfoVo tourBookInfoVo = MyModelUtil.copyTo(tourBookInfo, TourBookInfoVo.class);
        return ResponseResult.success(tourBookInfoVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourBookInfo originalTourBookInfo = tourBookInfoService.getById(id);
        if (originalTourBookInfo == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourBookInfoService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
