package com.tourism.webadmin.back.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.github.pagehelper.page.PageMethod;
import com.tourism.webadmin.back.vo.*;
import com.tourism.webadmin.back.dto.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.common.core.object.*;
import com.tourism.common.core.util.*;
import com.tourism.common.core.constant.*;
import com.tourism.common.core.annotation.MyRequestBody;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 * 旅游富文本操作控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "旅游富文本管理接口")
@Slf4j
@RestController
@RequestMapping("/admin/app/tourismContent")
public class TourismContentController {

    @Autowired
    private TourismContentService tourismContentService;

    /**
     * 新增旅游富文本数据。
     *
     * @param tourismContentDto 新增对象。
     * @return 应答结果对象，包含新增对象主键Id。
     */
    @ApiOperationSupport(ignoreParameters = {"tourismContentDto.id"})
    @SaCheckPermission("tourismContent.add")
    @OperationLog(type = SysOperationLogType.ADD)
    @PostMapping("/add")
    public ResponseResult<Long> add(@MyRequestBody TourismContentDto tourismContentDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourismContentDto, false);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourismContent tourismContent = MyModelUtil.copyTo(tourismContentDto, TourismContent.class);
        tourismContent = tourismContentService.saveNew(tourismContent);
        return ResponseResult.success(tourismContent.getId());
    }

    /**
     * 更新旅游富文本数据。
     *
     * @param tourismContentDto 更新对象。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourismContent.update")
    @OperationLog(type = SysOperationLogType.UPDATE)
    @PostMapping("/update")
    public ResponseResult<Void> update(@MyRequestBody TourismContentDto tourismContentDto) {
        String errorMessage = MyCommonUtil.getModelValidationError(tourismContentDto, true);
        if (errorMessage != null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_VALIDATED_FAILED, errorMessage);
        }
        TourismContent tourismContent = MyModelUtil.copyTo(tourismContentDto, TourismContent.class);
        TourismContent originalTourismContent = tourismContentService.getById(tourismContent.getId());
        if (originalTourismContent == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [数据] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourismContentService.update(tourismContent, originalTourismContent)) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        return ResponseResult.success();
    }

    /**
     * 删除旅游富文本数据。
     *
     * @param id 删除对象主键Id。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourismContent.delete")
    @OperationLog(type = SysOperationLogType.DELETE)
    @PostMapping("/delete")
    public ResponseResult<Void> delete(@MyRequestBody Long id) {
        if (MyCommonUtil.existBlankArgument(id)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        return this.doDelete(id);
    }

    /**
     * 批量删除旅游富文本数据。
     *
     * @param idList 待删除对象的主键Id列表。
     * @return 应答结果对象。
     */
    @SaCheckPermission("tourismContent.delete")
    @OperationLog(type = SysOperationLogType.DELETE_BATCH)
    @PostMapping("/deleteBatch")
    public ResponseResult<Void> deleteBatch(@MyRequestBody List<Long> idList) {
        if (MyCommonUtil.existBlankArgument(idList)) {
            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST);
        }
        for (Long id : idList) {
            ResponseResult<Void> responseResult = this.doDelete(id);
            if (!responseResult.isSuccess()) {
                return responseResult;
            }
        }
        return ResponseResult.success();
    }

    /**
     * 列出符合过滤条件的旅游富文本列表。
     *
     * @param tourismContentDtoFilter 过滤对象。
     * @param orderParam 排序参数。
     * @param pageParam 分页参数。
     * @return 应答结果对象，包含查询结果集。
     */
    @SaCheckPermission("tourismContent.view")
    @PostMapping("/list")
    public ResponseResult<MyPageData<TourismContentVo>> list(
            @MyRequestBody TourismContentDto tourismContentDtoFilter,
            @MyRequestBody MyOrderParam orderParam,
            @MyRequestBody MyPageParam pageParam) {
        if (pageParam != null) {
            PageMethod.startPage(pageParam.getPageNum(), pageParam.getPageSize(), pageParam.getCount());
        }
        TourismContent tourismContentFilter = MyModelUtil.copyTo(tourismContentDtoFilter, TourismContent.class);
        String orderBy = MyOrderParam.buildOrderBy(orderParam, TourismContent.class);
        List<TourismContent> tourismContentList =
                tourismContentService.getTourismContentListWithRelation(tourismContentFilter, orderBy);
        return ResponseResult.success(MyPageUtil.makeResponseData(tourismContentList, TourismContentVo.class));
    }

    /**
     * 查看指定旅游富文本对象详情。
     *
     * @param id 指定对象主键Id。
     * @return 应答结果对象，包含对象详情。
     */
    @SaCheckPermission("tourismContent.view")
    @GetMapping("/view")
    public ResponseResult<TourismContentVo> view(@RequestParam Long id) {
        TourismContent tourismContent = tourismContentService.getByIdWithRelation(id, MyRelationParam.full());
        if (tourismContent == null) {
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST);
        }
        TourismContentVo tourismContentVo = MyModelUtil.copyTo(tourismContent, TourismContentVo.class);
        return ResponseResult.success(tourismContentVo);
    }

    private ResponseResult<Void> doDelete(Long id) {
        String errorMessage;
        // 验证关联Id的数据合法性
        TourismContent originalTourismContent = tourismContentService.getById(id);
        if (originalTourismContent == null) {
            // NOTE: 修改下面方括号中的话述
            errorMessage = "数据验证失败，当前 [对象] 并不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        if (!tourismContentService.remove(id)) {
            errorMessage = "数据操作失败，删除的对象不存在，请刷新后重试！";
            return ResponseResult.error(ErrorCodeEnum.DATA_NOT_EXIST, errorMessage);
        }
        return ResponseResult.success();
    }
}
