package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * banner管理数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("bannerInfoService")
public class BannerInfoServiceImpl extends BaseService<BannerInfo, Long> implements BannerInfoService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private BannerInfoMapper bannerInfoMapper;
    @Autowired
    private DirectoryInfoService directoryInfoService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<BannerInfo> mapper() {
        return bannerInfoMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public BannerInfo saveNew(BannerInfo bannerInfo) {
        bannerInfoMapper.insert(this.buildDefaultValue(bannerInfo));
        return bannerInfo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<BannerInfo> bannerInfoList) {
        if (CollUtil.isNotEmpty(bannerInfoList)) {
            bannerInfoList.forEach(this::buildDefaultValue);
            bannerInfoMapper.insertList(bannerInfoList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<BannerInfo> bannerInfoList, int batchSize) {
        if (CollUtil.isEmpty(bannerInfoList)) {
            return;
        }
        if (batchSize <= 0) {
            batchSize = 10000;
        }
        int start = 0;
        do {
            int end = Math.min(bannerInfoList.size(), start + batchSize);
            List<BannerInfo> subList = bannerInfoList.subList(start, end);
            // 如果数据量过大，同时当前表中存在createTime或updateTime等字段，可以考虑在外部创建一次 new Date()，
            // 然后传入buildDefaultValue，这样可以减少对象的创建次数，降低GC，提升效率。橙单之所以没有这样生成，是因为
            // 有些业务场景下需要按照这两个日期字段排序，因此我们只是在这里给出优化建议。
            subList.forEach(this::buildDefaultValue);
            bannerInfoMapper.insertList(subList);
            if (end == bannerInfoList.size()) {
                break;
            }
            start += batchSize;
        } while (true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(BannerInfo bannerInfo, BannerInfo originalBannerInfo) {
        MyModelUtil.fillCommonsForUpdate(bannerInfo, originalBannerInfo);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<BannerInfo> uw = this.createUpdateQueryForNullValue(bannerInfo, bannerInfo.getId());
        return bannerInfoMapper.update(bannerInfo, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return bannerInfoMapper.deleteById(id) == 1;
    }

    @Override
    public List<BannerInfo> getBannerInfoList(BannerInfo filter, String orderBy) {
        return bannerInfoMapper.getBannerInfoList(filter, orderBy);
    }

    @Override
    public List<BannerInfo> getBannerInfoListWithRelation(BannerInfo bannerInfoFilter, String orderBy) {
        List<BannerInfo> resultList = bannerInfoMapper.getBannerInfoList(bannerInfoFilter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyImportList(List<BannerInfo> dataList, Set<String> ignoreFieldSet) {
        CallResult callResult;
        if (!CollUtil.contains(ignoreFieldSet, "enable")) {
            callResult = verifyImportForConstDict(dataList, "enableDictMap", BannerInfo::getEnable);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "mainBanner")) {
            callResult = verifyImportForConstDict(dataList, "mainBannerDictMap", BannerInfo::getMainBanner);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "belongTab")) {
            callResult = verifyImportForDatasourceDict(dataList, "belongTabDictMap", BannerInfo::getBelongTab);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        return CallResult.ok();
    }

    @Override
    public CallResult verifyRelatedData(BannerInfo bannerInfo, BannerInfo originalBannerInfo) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是基于字典的验证。
        if (this.needToVerify(bannerInfo, originalBannerInfo, BannerInfo::getBelongTab)
                && !directoryInfoService.existId(bannerInfo.getBelongTab())) {
            return CallResult.error(String.format(errorMessageFormat, "所属分类"));
        }
        return CallResult.ok();
    }

    private BannerInfo buildDefaultValue(BannerInfo bannerInfo) {
        if (bannerInfo.getId() == null) {
            bannerInfo.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(bannerInfo);
        bannerInfo.setDataState(GlobalDeletedFlag.NORMAL);
        return bannerInfo;
    }
}
