package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.TokenData;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 公司信息管理数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("companyInfoService")
public class CompanyInfoServiceImpl extends BaseService<CompanyInfo, Long> implements CompanyInfoService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private CompanyInfoMapper companyInfoMapper;
    @Autowired
    private CompanyContentService companyContentService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<CompanyInfo> mapper() {
        return companyInfoMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public CompanyInfo saveNew(CompanyInfo companyInfo) {
        companyInfoMapper.insert(this.buildDefaultValue(companyInfo));
        return companyInfo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<CompanyInfo> companyInfoList) {
        if (CollUtil.isNotEmpty(companyInfoList)) {
            companyInfoList.forEach(this::buildDefaultValue);
            companyInfoMapper.insertList(companyInfoList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<CompanyInfo> companyInfoList, int batchSize) {
        if (CollUtil.isEmpty(companyInfoList)) {
            return;
        }
        if (batchSize <= 0) {
            batchSize = 10000;
        }
        int start = 0;
        do {
            int end = Math.min(companyInfoList.size(), start + batchSize);
            List<CompanyInfo> subList = companyInfoList.subList(start, end);
            // 如果数据量过大，同时当前表中存在createTime或updateTime等字段，可以考虑在外部创建一次 new Date()，
            // 然后传入buildDefaultValue，这样可以减少对象的创建次数，降低GC，提升效率。橙单之所以没有这样生成，是因为
            // 有些业务场景下需要按照这两个日期字段排序，因此我们只是在这里给出优化建议。
            subList.forEach(this::buildDefaultValue);
            companyInfoMapper.insertList(subList);
            if (end == companyInfoList.size()) {
                break;
            }
            start += batchSize;
        } while (true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public CompanyInfo saveNewWithRelation(CompanyInfo companyInfo, JSONObject relationData) {
        this.saveNew(companyInfo);
        this.saveOrUpdateOneToOneRelationData(companyInfo, relationData);
        return companyInfo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(CompanyInfo companyInfo, CompanyInfo originalCompanyInfo) {
        companyInfo.setCreateUserId(originalCompanyInfo.getCreateUserId());
        companyInfo.setCreateTime(originalCompanyInfo.getCreateTime());
        companyInfo.setUpdateTime(new Date());
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<CompanyInfo> uw = this.createUpdateQueryForNullValue(companyInfo, companyInfo.getId());
        return companyInfoMapper.update(companyInfo, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateWithRelation(
            CompanyInfo companyInfo, CompanyInfo originalCompanyInfo, JSONObject relationData) {
        // companyInfo 为空的时候，无需修改主表数据。
        if (companyInfo != null && !this.update(companyInfo, originalCompanyInfo)) {
            return false;
        }
        this.saveOrUpdateOneToOneRelationData(originalCompanyInfo, relationData);
        return true;
    }

    private void saveOrUpdateOneToOneRelationData(CompanyInfo companyInfo, JSONObject relationData) {
        // 对于一对一新增或更新，如果主键值为空就新增，否则就更新，同时更新updateTime和updateUserId。
        CompanyContent companyContent = relationData.getObject("companyContent", CompanyContent.class);
        if (companyContent != null) {
            companyContent.setAssociationId(companyInfo.getId());
            companyContentService.saveNewOrUpdate(companyContent,
                    companyContentService::saveNew, companyContentService::update);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        if (companyInfoMapper.deleteById(id) == 0) {
            return false;
        }
        companyContentService.removeByAssociationId(id);
        return true;
    }

    @Override
    public List<CompanyInfo> getCompanyInfoList(CompanyInfo filter, String orderBy) {
        return companyInfoMapper.getCompanyInfoList(filter, orderBy);
    }

    @Override
    public List<CompanyInfo> getCompanyInfoListWithRelation(CompanyInfo filter, String orderBy) {
        List<CompanyInfo> resultList = companyInfoMapper.getCompanyInfoList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyImportList(List<CompanyInfo> dataList, Set<String> ignoreFieldSet) {
        CallResult callResult;
        if (!CollUtil.contains(ignoreFieldSet, "companyType")) {
            callResult = verifyImportForConstDict(dataList, "companyTypeDictMap", CompanyInfo::getCompanyType);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "id")) {
            callResult = verifyImportForOneToOneRelation(dataList, "CompanyContent", CompanyInfo::getId);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        return CallResult.ok();
    }

    private CompanyInfo buildDefaultValue(CompanyInfo companyInfo) {
        if (companyInfo.getId() == null) {
            companyInfo.setId(idGenerator.nextLongId());
        }
        TokenData tokenData = TokenData.takeFromRequest();
        companyInfo.setCreateUserId(tokenData.getUserId());
        Date now = new Date();
        companyInfo.setCreateTime(now);
        companyInfo.setUpdateTime(now);
        companyInfo.setDataState(GlobalDeletedFlag.NORMAL);
        return companyInfo;
    }
}
