package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.dao.DeliveryAddressMapper;
import com.tourism.webadmin.back.model.DeliveryAddress;
import com.tourism.webadmin.back.service.DeliveryAddressService;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 外卖地址管理数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@MyDataSource(DataSourceType.MAIN)
@Service("deliveryAddressService")
public class DeliveryAddressServiceImpl extends BaseService<DeliveryAddress, Long> implements DeliveryAddressService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private DeliveryAddressMapper deliveryAddressMapper;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<DeliveryAddress> mapper() {
        return deliveryAddressMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public DeliveryAddress saveNew(DeliveryAddress deliveryAddress) {
        this.verifyMaskFieldData(deliveryAddress);
        deliveryAddressMapper.insert(this.buildDefaultValue(deliveryAddress));
        return deliveryAddress;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<DeliveryAddress> deliveryAddressList) {
        if (CollUtil.isNotEmpty(deliveryAddressList)) {
            deliveryAddressList.forEach(this::buildDefaultValue);
            deliveryAddressList.forEach(this::verifyMaskFieldData);
            deliveryAddressMapper.insertList(deliveryAddressList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<DeliveryAddress> deliveryAddressList, int batchSize) {
        if (CollUtil.isEmpty(deliveryAddressList)) {
            return;
        }
        if (batchSize <= 0) {
            batchSize = 10000;
        }
        int start = 0;
        do {
            int end = Math.min(deliveryAddressList.size(), start + batchSize);
            List<DeliveryAddress> subList = deliveryAddressList.subList(start, end);
            // 如果数据量过大，同时当前表中存在createTime或updateTime等字段，可以考虑在外部创建一次 new Date()，
            // 然后传入buildDefaultValue，这样可以减少对象的创建次数，降低GC，提升效率。橙单之所以没有这样生成，是因为
            // 有些业务场景下需要按照这两个日期字段排序，因此我们只是在这里给出优化建议。
            subList.forEach(this::buildDefaultValue);
            deliveryAddressMapper.insertList(subList);
            if (end == deliveryAddressList.size()) {
                break;
            }
            start += batchSize;
        } while (true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(DeliveryAddress deliveryAddress, DeliveryAddress originalDeliveryAddress) {
        this.compareAndSetMaskFieldData(deliveryAddress, originalDeliveryAddress);
        MyModelUtil.fillCommonsForUpdate(deliveryAddress, originalDeliveryAddress);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<DeliveryAddress> uw = this.createUpdateQueryForNullValue(deliveryAddress, deliveryAddress.getId());
        return deliveryAddressMapper.update(deliveryAddress, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return deliveryAddressMapper.deleteById(id) == 1;
    }

    @Override
    public List<DeliveryAddress> getDeliveryAddressList(DeliveryAddress filter, String orderBy) {
        return deliveryAddressMapper.getDeliveryAddressList(filter, orderBy);
    }

    @Override
    public List<DeliveryAddress> getDeliveryAddressListWithRelation(DeliveryAddress filter, String orderBy) {
        List<DeliveryAddress> resultList = deliveryAddressMapper.getDeliveryAddressList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    private DeliveryAddress buildDefaultValue(DeliveryAddress deliveryAddress) {
        if (deliveryAddress.getId() == null) {
            deliveryAddress.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(deliveryAddress);
        deliveryAddress.setDataState(GlobalDeletedFlag.NORMAL);
        MyModelUtil.setDefaultValue(deliveryAddress, "address", "");
        return deliveryAddress;
    }
}
