package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 劳务项目管理数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("jobProjectService")
public class JobProjectServiceImpl extends BaseService<JobProject, Long> implements JobProjectService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private JobProjectMapper jobProjectMapper;
    @Autowired
    private DirectoryInfoService directoryInfoService;
    @Autowired
    private JobFileService jobFileService;
    @Autowired
    private JobContentService jobContentService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<JobProject> mapper() {
        return jobProjectMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public JobProject saveNew(JobProject jobProject) {
        jobProjectMapper.insert(this.buildDefaultValue(jobProject));
        return jobProject;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<JobProject> jobProjectList) {
        if (CollUtil.isNotEmpty(jobProjectList)) {
            jobProjectList.forEach(this::buildDefaultValue);
            jobProjectMapper.insertList(jobProjectList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<JobProject> jobProjectList, int batchSize) {
        if (CollUtil.isEmpty(jobProjectList)) {
            return;
        }
        if (batchSize <= 0) {
            batchSize = 10000;
        }
        int start = 0;
        do {
            int end = Math.min(jobProjectList.size(), start + batchSize);
            List<JobProject> subList = jobProjectList.subList(start, end);
            // 如果数据量过大，同时当前表中存在createTime或updateTime等字段，可以考虑在外部创建一次 new Date()，
            // 然后传入buildDefaultValue，这样可以减少对象的创建次数，降低GC，提升效率。橙单之所以没有这样生成，是因为
            // 有些业务场景下需要按照这两个日期字段排序，因此我们只是在这里给出优化建议。
            subList.forEach(this::buildDefaultValue);
            jobProjectMapper.insertList(subList);
            if (end == jobProjectList.size()) {
                break;
            }
            start += batchSize;
        } while (true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public JobProject saveNewWithRelation(JobProject jobProject, JSONObject relationData) {
        this.saveNew(jobProject);
        this.saveOrUpdateOneToOneRelationData(jobProject, relationData);
        return jobProject;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(JobProject jobProject, JobProject originalJobProject) {
        MyModelUtil.fillCommonsForUpdate(jobProject, originalJobProject);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<JobProject> uw = this.createUpdateQueryForNullValue(jobProject, jobProject.getId());
        return jobProjectMapper.update(jobProject, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateWithRelation(
            JobProject jobProject, JobProject originalJobProject, JSONObject relationData) {
        // jobProject 为空的时候，无需修改主表数据。
        if (jobProject != null && !this.update(jobProject, originalJobProject)) {
            return false;
        }
        this.saveOrUpdateOneToOneRelationData(originalJobProject, relationData);
        return true;
    }

    private void saveOrUpdateOneToOneRelationData(JobProject jobProject, JSONObject relationData) {
        // 对于一对一新增或更新，如果主键值为空就新增，否则就更新，同时更新updateTime和updateUserId。
        JobFile jobFile = relationData.getObject("jobFile", JobFile.class);
        if (jobFile != null) {
            jobFile.setAssociationTableId(jobProject.getId());
            jobFileService.saveNewOrUpdate(jobFile,
                    jobFileService::saveNew, jobFileService::update);
        }
        JobContent jobContent = relationData.getObject("jobContent", JobContent.class);
        if (jobContent != null) {
            jobContent.setAssociationId(jobProject.getId());
            jobContentService.saveNewOrUpdate(jobContent,
                    jobContentService::saveNew, jobContentService::update);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        if (jobProjectMapper.deleteById(id) == 0) {
            return false;
        }
        jobFileService.removeByAssociationTableId(id);
        jobContentService.removeByAssociationId(id);
        return true;
    }

    @Override
    public List<JobProject> getJobProjectList(JobProject filter, String orderBy) {
        return jobProjectMapper.getJobProjectList(filter, orderBy);
    }

    @Override
    public List<JobProject> getJobProjectListWithRelation(JobProject filter, String orderBy) {
        List<JobProject> resultList = jobProjectMapper.getJobProjectList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.dictOnly(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyImportList(List<JobProject> dataList, Set<String> ignoreFieldSet) {
        CallResult callResult;
        if (!CollUtil.contains(ignoreFieldSet, "isHotspot")) {
            callResult = verifyImportForConstDict(dataList, "isHotspotDictMap", JobProject::getIsHotspot);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "enable")) {
            callResult = verifyImportForConstDict(dataList, "enableDictMap", JobProject::getEnable);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "belongTab")) {
            callResult = verifyImportForDatasourceDict(dataList, "belongTabDictMap", JobProject::getBelongTab);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        if (!CollUtil.contains(ignoreFieldSet, "id")) {
            callResult = verifyImportForOneToOneRelation(dataList, "JobFile", JobProject::getId);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        return CallResult.ok();
    }

    @Override
    public CallResult verifyRelatedData(JobProject jobProject, JobProject originalJobProject) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是基于字典的验证。
        if (this.needToVerify(jobProject, originalJobProject, JobProject::getBelongTab)
                && !directoryInfoService.existId(jobProject.getBelongTab())) {
            return CallResult.error(String.format(errorMessageFormat, "所属分类"));
        }
        return CallResult.ok();
    }

    private JobProject buildDefaultValue(JobProject jobProject) {
        if (jobProject.getId() == null) {
            jobProject.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(jobProject);
        jobProject.setDataState(GlobalDeletedFlag.NORMAL);
        return jobProject;
    }
}
