package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ReflectUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.common.core.object.TokenData;
import com.tourism.webadmin.app.wechat.model.WechatRestaurantCart;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.lang.reflect.Field;
import java.util.*;

/**
 * 外卖购物车数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("restaurantCartService")
public class RestaurantCartServiceImpl extends BaseService<RestaurantCart, Long> implements RestaurantCartService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private RestaurantCartMapper restaurantCartMapper;
    /**
     * 整个工程的实体对象中，创建者Id字段的Java对象名。
     */
    public static final String CREATE_USER_ID_FIELD_NAME = "createUserId";
    /**
     * 整个工程的实体对象中，创建时间字段的Java对象名。
     */
    public static final String CREATE_TIME_FIELD_NAME = "createTime";
    /**
     * 整个工程的实体对象中，更新者Id字段的Java对象名。
     */
    public static final String UPDATE_USER_ID_FIELD_NAME = "updateUserId";
    /**
     * 整个工程的实体对象中，更新时间字段的Java对象名。
     */
    public static final String UPDATE_TIME_FIELD_NAME = "updateTime";

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<RestaurantCart> mapper() {
        return restaurantCartMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public RestaurantCart saveNew(RestaurantCart restaurantCart) {
        restaurantCartMapper.insert(this.buildDefaultValue(restaurantCart));
        return restaurantCart;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<RestaurantCart> restaurantCartList) {
        if (CollUtil.isNotEmpty(restaurantCartList)) {
            restaurantCartList.forEach(this::buildDefaultValue);
            restaurantCartMapper.insertList(restaurantCartList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<RestaurantCart> restaurantCartList, int batchSize) {
        if (CollUtil.isEmpty(restaurantCartList)) {
            return;
        }
        if (batchSize <= 0) {
            batchSize = 10000;
        }
        int start = 0;
        do {
            int end = Math.min(restaurantCartList.size(), start + batchSize);
            List<RestaurantCart> subList = restaurantCartList.subList(start, end);
            // 如果数据量过大，同时当前表中存在createTime或updateTime等字段，可以考虑在外部创建一次 new Date()，
            // 然后传入buildDefaultValue，这样可以减少对象的创建次数，降低GC，提升效率。橙单之所以没有这样生成，是因为
            // 有些业务场景下需要按照这两个日期字段排序，因此我们只是在这里给出优化建议。
            subList.forEach(this::buildDefaultValue);
            restaurantCartMapper.insertList(subList);
            if (end == restaurantCartList.size()) {
                break;
            }
            start += batchSize;
        } while (true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(RestaurantCart restaurantCart, RestaurantCart originalRestaurantCart) {
//        MyModelUtil.fillCommonsForUpdate(restaurantCart, originalRestaurantCart);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<RestaurantCart> uw = this.createUpdateQueryForNullValue(restaurantCart, restaurantCart.getId());
        return restaurantCartMapper.update(restaurantCart, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return restaurantCartMapper.deleteById(id) == 1;
    }

    @Override
    public List<RestaurantCart> getRestaurantCartList(RestaurantCart filter, String orderBy) {
        return restaurantCartMapper.getRestaurantCartList(filter, orderBy);
    }

    @Override
    public List<RestaurantCart> getRestaurantCartListWithRelation(RestaurantCart filter, String orderBy) {
        List<RestaurantCart> resultList = restaurantCartMapper.getRestaurantCartList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    private RestaurantCart buildDefaultValue(RestaurantCart restaurantCart) {
        if (restaurantCart.getId() == null) {
            restaurantCart.setId(idGenerator.nextLongId());
        }
//        MyModelUtil.fillCommonsForInsert(restaurantCart);
        restaurantCart.setDeletedFlag(GlobalDeletedFlag.NORMAL);
        return restaurantCart;
    }


    /**
     * 获取单表查询结果。由于没有关联数据查询，因此在仅仅获取单表数据的场景下，效率更高。
     * 如果需要同时获取关联数据，请移步(getRestaurantCartListWithRelation)方法。
     *
     * @param filter  过滤对象。
     * @param orderBy 排序参数。
     * @return 查询结果集。
     */
    @Override
    public List<WechatRestaurantCart> getRestaurantCartListAndFoodDetail(WechatRestaurantCart filter, String orderBy){
        List<WechatRestaurantCart> resultList = restaurantCartMapper.getRestaurantCartListAndFoodDetail(filter, orderBy);
        return resultList;
    }

}
