package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.dao.*;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 餐馆类型数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("restaurantTypeService")
public class RestaurantTypeServiceImpl extends BaseService<RestaurantType, Long> implements RestaurantTypeService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private RestaurantTypeMapper restaurantTypeMapper;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<RestaurantType> mapper() {
        return restaurantTypeMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public RestaurantType saveNew(RestaurantType restaurantType) {
        restaurantTypeMapper.insert(this.buildDefaultValue(restaurantType));
        return restaurantType;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<RestaurantType> restaurantTypeList) {
        if (CollUtil.isNotEmpty(restaurantTypeList)) {
            restaurantTypeList.forEach(this::buildDefaultValue);
            restaurantTypeMapper.insertList(restaurantTypeList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<RestaurantType> restaurantTypeList, int batchSize) {
        if (CollUtil.isEmpty(restaurantTypeList)) {
            return;
        }
        if (batchSize <= 0) {
            batchSize = 10000;
        }
        int start = 0;
        do {
            int end = Math.min(restaurantTypeList.size(), start + batchSize);
            List<RestaurantType> subList = restaurantTypeList.subList(start, end);
            // 如果数据量过大，同时当前表中存在createTime或updateTime等字段，可以考虑在外部创建一次 new Date()，
            // 然后传入buildDefaultValue，这样可以减少对象的创建次数，降低GC，提升效率。橙单之所以没有这样生成，是因为
            // 有些业务场景下需要按照这两个日期字段排序，因此我们只是在这里给出优化建议。
            subList.forEach(this::buildDefaultValue);
            restaurantTypeMapper.insertList(subList);
            if (end == restaurantTypeList.size()) {
                break;
            }
            start += batchSize;
        } while (true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(RestaurantType restaurantType, RestaurantType originalRestaurantType) {
        MyModelUtil.fillCommonsForUpdate(restaurantType, originalRestaurantType);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<RestaurantType> uw = this.createUpdateQueryForNullValue(restaurantType, restaurantType.getId());
        return restaurantTypeMapper.update(restaurantType, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return restaurantTypeMapper.deleteById(id) == 1;
    }

    @Override
    public List<RestaurantType> getRestaurantTypeList(RestaurantType filter, String orderBy) {
        return restaurantTypeMapper.getRestaurantTypeList(filter, orderBy);
    }

    @Override
    public List<RestaurantType> getRestaurantTypeListWithRelation(RestaurantType filter, String orderBy) {
        List<RestaurantType> resultList = restaurantTypeMapper.getRestaurantTypeList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyImportList(List<RestaurantType> dataList, Set<String> ignoreFieldSet) {
        CallResult callResult;
        if (!CollUtil.contains(ignoreFieldSet, "enable")) {
            callResult = verifyImportForConstDict(dataList, "enableDictMap", RestaurantType::getEnable);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        return CallResult.ok();
    }

    private RestaurantType buildDefaultValue(RestaurantType restaurantType) {
        if (restaurantType.getId() == null) {
            restaurantType.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(restaurantType);
        restaurantType.setDataState(GlobalDeletedFlag.NORMAL);
        return restaurantType;
    }
}
