package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.webadmin.back.dao.TourBookInfoMapper;
import com.tourism.webadmin.back.model.TourBookInfo;
import com.tourism.webadmin.back.service.TourBookInfoService;
import com.tourism.webadmin.config.DataSourceType;
import com.tourism.common.core.annotation.MyDataSource;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.tourism.common.dict.service.GlobalDictService;
import com.github.pagehelper.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 门户预定管理数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@MyDataSource(DataSourceType.MAIN)
@Service("tourBookInfoService")
public class TourBookInfoServiceImpl extends BaseService<TourBookInfo, Long> implements TourBookInfoService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourBookInfoMapper tourBookInfoMapper;
    @Autowired
    private GlobalDictService globalDictService;

    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourBookInfo> mapper() {
        return tourBookInfoMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourBookInfo saveNew(TourBookInfo tourBookInfo) {
        tourBookInfoMapper.insert(this.buildDefaultValue(tourBookInfo));
        return tourBookInfo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourBookInfo> tourBookInfoList) {
        if (CollUtil.isNotEmpty(tourBookInfoList)) {
            tourBookInfoList.forEach(this::buildDefaultValue);
            tourBookInfoMapper.insertList(tourBookInfoList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourBookInfo> tourBookInfoList, int batchSize) {
        if (CollUtil.isEmpty(tourBookInfoList)) {
            return;
        }
        if (batchSize <= 0) {
            batchSize = 10000;
        }
        int start = 0;
        do {
            int end = Math.min(tourBookInfoList.size(), start + batchSize);
            List<TourBookInfo> subList = tourBookInfoList.subList(start, end);
            // 如果数据量过大，同时当前表中存在createTime或updateTime等字段，可以考虑在外部创建一次 new Date()，
            // 然后传入buildDefaultValue，这样可以减少对象的创建次数，降低GC，提升效率。橙单之所以没有这样生成，是因为
            // 有些业务场景下需要按照这两个日期字段排序，因此我们只是在这里给出优化建议。
            subList.forEach(this::buildDefaultValue);
            tourBookInfoMapper.insertList(subList);
            if (end == tourBookInfoList.size()) {
                break;
            }
            start += batchSize;
        } while (true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourBookInfo tourBookInfo, TourBookInfo originalTourBookInfo) {
        MyModelUtil.fillCommonsForUpdate(tourBookInfo, originalTourBookInfo);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourBookInfo> uw = this.createUpdateQueryForNullValue(tourBookInfo, tourBookInfo.getId());
        return tourBookInfoMapper.update(tourBookInfo, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long id) {
        return tourBookInfoMapper.deleteById(id) == 1;
    }

    @Override
    public List<TourBookInfo> getTourBookInfoList(TourBookInfo filter, String orderBy) {
        return tourBookInfoMapper.getTourBookInfoList(filter, orderBy);
    }

    @Override
    public List<TourBookInfo> getTourBookInfoListWithRelation(TourBookInfo filter, String orderBy) {
        List<TourBookInfo> resultList = tourBookInfoMapper.getTourBookInfoList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyImportList(List<TourBookInfo> dataList, Set<String> ignoreFieldSet) {
        CallResult callResult;
        if (!CollUtil.contains(ignoreFieldSet, "isHandle")) {
            callResult = verifyImportForGlobalDict(dataList, "isHandleDictMap", TourBookInfo::getIsHandle);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        return CallResult.ok();
    }

    @Override
    public CallResult verifyRelatedData(TourBookInfo tourBookInfo, TourBookInfo originalTourBookInfo) {
        String errorMessageFormat = "数据验证失败，关联的%s并不存在，请刷新后重试！";
        //这里是基于字典的验证。
        if (this.needToVerify(tourBookInfo, originalTourBookInfo, TourBookInfo::getIsHandle)
                && !globalDictService.existDictItemFromCache("IsHandle", tourBookInfo.getIsHandle())) {
            return CallResult.error(String.format(errorMessageFormat, "是否处理(0:未处理 1:已处理)"));
        }
        return CallResult.ok();
    }

    private TourBookInfo buildDefaultValue(TourBookInfo tourBookInfo) {
        if (tourBookInfo.getId() == null) {
            tourBookInfo.setId(idGenerator.nextLongId());
        }
        MyModelUtil.fillCommonsForInsert(tourBookInfo);
        tourBookInfo.setDeletedFlag(GlobalDeletedFlag.NORMAL);
        return tourBookInfo;
    }
}
