package com.tourism.webadmin.back.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ReflectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.tourism.common.core.base.dao.BaseDaoMapper;
import com.tourism.common.core.constant.GlobalDeletedFlag;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.CallResult;
import com.tourism.common.core.base.service.BaseService;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.sequence.wrapper.IdGeneratorWrapper;
import com.github.pagehelper.Page;
import com.tourism.webadmin.back.dao.TourUserMapper;
import com.tourism.webadmin.back.model.TourUser;
import com.tourism.webadmin.back.service.TourUserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.lang.reflect.Field;
import java.util.*;

/**
 * 门户网站用户管理数据操作服务类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Slf4j
@Service("tourUserService")
public class TourUserServiceImpl extends BaseService<TourUser, Long> implements TourUserService {

    @Autowired
    private IdGeneratorWrapper idGenerator;
    @Autowired
    private TourUserMapper tourUserMapper;
    @Autowired
    private PasswordEncoder passwordEncoder;
    /**
     * 整个工程的实体对象中，创建者Id字段的Java对象名。
     */
    public static final String CREATE_USER_ID_FIELD_NAME = "createUserId";
    /**
     * 整个工程的实体对象中，创建时间字段的Java对象名。
     */
    public static final String CREATE_TIME_FIELD_NAME = "createTime";
    /**
     * 整个工程的实体对象中，更新者Id字段的Java对象名。
     */
    public static final String UPDATE_USER_ID_FIELD_NAME = "updateUserId";
    /**
     * 整个工程的实体对象中，更新时间字段的Java对象名。
     */
    public static final String UPDATE_TIME_FIELD_NAME = "updateTime";
    /**
     * 整个工程的实体对象中，更新时间字段的Java对象名。
     * 用户注册的则把createUserId字段作为注册用户id
     */
    public static final Integer CREATE_USER_ID_FIELD_REGISTER_NAME = 0;
    /**
     * 返回当前Service的主表Mapper对象。
     *
     * @return 主表Mapper对象。
     */
    @Override
    protected BaseDaoMapper<TourUser> mapper() {
        return tourUserMapper;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public TourUser saveNew(TourUser tourUser) {
        tourUserMapper.insert(this.buildDefaultValue(tourUser));
        return tourUser;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourUser> tourUserList) {
        if (CollUtil.isNotEmpty(tourUserList)) {
            tourUserList.forEach(this::buildDefaultValue);
            tourUserMapper.insertList(tourUserList);
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveNewBatch(List<TourUser> tourUserList, int batchSize) {
        if (CollUtil.isEmpty(tourUserList)) {
            return;
        }
        if (batchSize <= 0) {
            batchSize = 10000;
        }
        int start = 0;
        do {
            int end = Math.min(tourUserList.size(), start + batchSize);
            List<TourUser> subList = tourUserList.subList(start, end);
            // 如果数据量过大，同时当前表中存在createTime或updateTime等字段，可以考虑在外部创建一次 new Date()，
            // 然后传入buildDefaultValue，这样可以减少对象的创建次数，降低GC，提升效率。橙单之所以没有这样生成，是因为
            // 有些业务场景下需要按照这两个日期字段排序，因此我们只是在这里给出优化建议。
            subList.forEach(this::buildDefaultValue);
            tourUserMapper.insertList(subList);
            if (end == tourUserList.size()) {
                break;
            }
            start += batchSize;
        } while (true);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean update(TourUser tourUser, TourUser originalTourUser) {
        MyModelUtil.fillCommonsForUpdate(tourUser, originalTourUser);
        // 这里重点提示，在执行主表数据更新之前，如果有哪些字段不支持修改操作，请用原有数据对象字段替换当前数据字段。
        UpdateWrapper<TourUser> uw = this.createUpdateQueryForNullValue(tourUser, tourUser.getUserId());
        return tourUserMapper.update(tourUser, uw) == 1;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean remove(Long userId) {
        return tourUserMapper.deleteById(userId) == 1;
    }

    @Override
    public List<TourUser> getTourUserList(TourUser filter, String orderBy) {
        return tourUserMapper.getTourUserList(filter, orderBy);
    }

    @Override
    public List<TourUser> getTourUserListWithRelation(TourUser filter, String orderBy) {
        List<TourUser> resultList = tourUserMapper.getTourUserList(filter, orderBy);
        // 在缺省生成的代码中，如果查询结果resultList不是Page对象，说明没有分页，那么就很可能是数据导出接口调用了当前方法。
        // 为了避免一次性的大量数据关联，规避因此而造成的系统运行性能冲击，这里手动进行了分批次读取，开发者可按需修改该值。
        int batchSize = resultList instanceof Page ? 0 : 1000;
        this.buildRelationForDataList(resultList, MyRelationParam.normal(), batchSize);
        return resultList;
    }

    @Override
    public CallResult verifyImportList(List<TourUser> dataList, Set<String> ignoreFieldSet) {
        CallResult callResult;
        if (!CollUtil.contains(ignoreFieldSet, "userStatus")) {
            callResult = verifyImportForConstDict(dataList, "userStatusDictMap", TourUser::getUserStatus);
            if (!callResult.isSuccess()) {
                return callResult;
            }
        }
        return CallResult.ok();
    }

    private TourUser buildDefaultValue(TourUser tourUser) {
        if (tourUser.getUserId() == null) {
            tourUser.setUserId(idGenerator.nextLongId());
        }
        fillCommonsForInsert(tourUser);
        tourUser.setDeletedFlag(GlobalDeletedFlag.NORMAL);
        return tourUser;
    }

    /**
     * 获取指定登录名的用户对象。
     *
     * @param loginName 指定登录用户名。
     * @return 用户对象。
     */
    @Override
    public TourUser getTourUserByLoginName(String loginName){
        TourUser filter = new TourUser();
        filter.setLoginName(loginName);
        return tourUserMapper.selectOne(new QueryWrapper<>(filter));
    }
    /**
     * 获取指定登录名的用户对象。
     *
     * @param loginMoblie 指定登录手机号。
     * @return 用户对象。
     */
    @Override
    public TourUser getTourUserByLoginMobile(String loginMoblie){
        TourUser filter = new TourUser();
        filter.setMobile(loginMoblie);
        return tourUserMapper.selectOne(new QueryWrapper<>(filter));
    }

    /**
     * 修改用户密码。
     * @param tourUserId  用户主键Id。
     * @param newPass 新密码。
     * @return 成功返回true，否则false。
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean changePassword(Long tourUserId, String newPass) {
        TourUser updatedTourUser = new TourUser();
        updatedTourUser.setUserId(tourUserId);
        updatedTourUser.setPassword(passwordEncoder.encode(newPass));
        return tourUserMapper.updateById(updatedTourUser) == 1;
    }

    /**
     * 修改用户头像。
     *
     * @param tourUserId  用户主键Id。
     * @param newHeadImage 新的头像信息。
     * @return 成功返回true，否则false。
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean changeHeadImage(Long tourUserId, String newHeadImage) {
        TourUser updatedUser = new TourUser();
        updatedUser.setUserId(tourUserId);
        updatedUser.setHeadImageUrl(newHeadImage);
        return tourUserMapper.updateById(updatedUser) == 1;
    }

    /**
     * 在插入实体对象数据之前，可以调用该方法，初始化通用字段的数据。
     *
     * @param data 实体对象。
     * @param <M>  实体对象类型。
     */
    public static <M> void fillCommonsForInsert(M data) {
        Field createdByField = ReflectUtil.getField(data.getClass(), CREATE_USER_ID_FIELD_NAME);
        if (createdByField != null) {
            ReflectUtil.setFieldValue(data, createdByField, CREATE_USER_ID_FIELD_REGISTER_NAME);
        }
        Field createTimeField = ReflectUtil.getField(data.getClass(), CREATE_TIME_FIELD_NAME);
        if (createTimeField != null) {
            ReflectUtil.setFieldValue(data, createTimeField, new Date());
        }
        Field updatedByField = ReflectUtil.getField(data.getClass(), UPDATE_USER_ID_FIELD_NAME);
        if (updatedByField != null) {
            ReflectUtil.setFieldValue(data, updatedByField, CREATE_USER_ID_FIELD_REGISTER_NAME);
        }
        Field updateTimeField = ReflectUtil.getField(data.getClass(), UPDATE_TIME_FIELD_NAME);
        if (updateTimeField != null) {
            ReflectUtil.setFieldValue(data, updateTimeField, new Date());
        }
    }
}
